@@ -57,13 +57,14 @@ from jax._src.config import (
   transfer_guard_host_to_device as transfer_guard_host_to_device,
   transfer_guard_device_to_device as transfer_guard_device_to_device,
   transfer_guard_device_to_host as transfer_guard_device_to_host,
+  spmd_mode as spmd_mode,
 )
 from .core import eval_context as ensure_compile_time_eval
 from jax._src.environment_info import print_environment_info as print_environment_info
 from jax._src.api import (
   ad,  # TODO(phawkins): update users to avoid this.
   effects_barrier,
-  block_until_ready,
+  block_until_ready as block_until_ready,
   checkpoint as checkpoint,
   checkpoint_policies as checkpoint_policies,
   clear_backends as clear_backends,
@@ -97,6 +98,7 @@ from jax._src.api import (
   local_devices as local_devices,
   linearize as linearize,
   linear_transpose as linear_transpose,
+  live_arrays as live_arrays,
   make_jaxpr as make_jaxpr,
   named_call as named_call,
   named_scope as named_scope,
@@ -149,6 +151,7 @@ from jax import numpy as numpy
 from jax import ops as ops
 from jax import profiler as profiler
 from jax import random as random
+from jax import scipy as scipy
 from jax import sharding as sharding
 from jax import stages as stages
 from jax import tree_util as tree_util
