@@ -13,12 +13,10 @@
 # limitations under the License.
 
 from functools import partial
-import operator as op
 from typing import (Callable, Optional, List, Tuple, Sequence, Set, Union, Any,
                     FrozenSet)
 import types
 
-from absl import logging
 import numpy as np
 
 import jax
@@ -31,7 +29,6 @@ from jax.interpreters import partial_eval as pe
 from jax.interpreters import xla
 from jax.tree_util import tree_flatten, tree_unflatten
 from jax._src import ad_util
-from jax._src import lax
 from jax._src import util
 from jax._src import source_info_util
 from jax._src import traceback_util
@@ -367,13 +364,14 @@ def _trace_to_jaxpr(fun, in_tree, in_avals):
 ### Utilities
 
 def saved_residuals(f, *args, **kwargs) -> List[Tuple[core.AbstractValue, str]]:
-  args, in_tree = tree_flatten((args, kwargs))
+  in_leaves, in_tree = tree_flatten((args, kwargs))
 
   def f_(*args):
     args, kwargs = tree_unflatten(in_tree, args)
     return f(*args, **kwargs)
 
-  jaxpr = jax.make_jaxpr(lambda *args: jax.linearize(f_, *args)[1])(*args).jaxpr
+  jaxpr = jax.make_jaxpr(lambda *args: jax.linearize(f_, *args)[1])(
+      *in_leaves).jaxpr
   res_lits = [x for x in jaxpr.outvars if     isinstance(x, core.Literal)]
   res_vars = {x for x in jaxpr.outvars if not isinstance(x, core.Literal)}
 
@@ -386,7 +384,7 @@ def saved_residuals(f, *args, **kwargs) -> List[Tuple[core.AbstractValue, str]]:
     if v in res_vars:
       results.append((v.aval, 'from a constant'))
 
-  assert len(jaxpr.invars) == len(args)
+  assert len(jaxpr.invars) == len(in_leaves)
   for i, v in enumerate(jaxpr.invars):
     if v in res_vars:
       src = f'from {pe.arg_info_pytree(f, in_tree, True, [i])}'
@@ -571,6 +569,8 @@ def remat_vmap(axis_size, axis_name, main_type, args, dims, *, jaxpr, **params):
       [batching.zero_if_mapped] * len(jaxpr.outvars),
       axis_name=axis_name, main_type=main_type)
   jaxpr_batched, consts = jaxpr_batched_.jaxpr, jaxpr_batched_.consts
+  if consts:
+    jaxpr_batched = pe.convert_constvars_jaxpr(jaxpr_batched)
   out_dims = [0 if b else None for b in out_batched]
   return remat_p.bind(*consts, *args, jaxpr=jaxpr_batched, **params), out_dims
 batching.axis_primitive_batchers[remat_p] = remat_vmap
