@@ -100,7 +100,7 @@ def _single_device_array_from_buf(buf, committed):
                    committed=committed, _skip_checks=True)
 
 
-@pxla.use_cpp_class(xc.ArrayImpl if xc._version >= 97 else None)
+@pxla.use_cpp_class(xc.ArrayImpl if xc._version >= 99 else None)
 class ArrayImpl(basearray.Array):
   # TODO(yashkatariya): Add __slots__ here.
 
@@ -407,6 +407,28 @@ class ArrayImpl(basearray.Array):
       out.append(Shard(db.device(), self.sharding, self.shape, array))
     return out
 
+  @property
+  def global_shards(self) -> Sequence[Shard]:
+    """Returns list of all `Shard`s of the Array across all devices.
+
+    The result includes shards that are not addressable by the current process.
+    If a `Shard` is not addressable, then its `data` will be `None`.
+    """
+    self._check_if_deleted()
+    if self.is_fully_addressable:  # pylint: disable=using-constant-test
+      return self.addressable_shards
+
+    out = []
+    device_id_to_buffer = {db.device().id: db for db in self._arrays}
+    for global_d in self.sharding.device_set:
+      if device_id_to_buffer.get(global_d.id, None) is not None:
+        array = _single_device_array_from_buf(
+            device_id_to_buffer[global_d.id], self._committed)
+      else:
+        array = None
+      out.append(Shard(global_d, self.sharding, self.shape, array))
+    return out
+
   def delete(self):
     if self._arrays is None:
       return
@@ -415,6 +437,7 @@ class ArrayImpl(basearray.Array):
     self._arrays = None
     self._npy_value = None
 
+  @pxla.use_cpp_method
   def is_deleted(self):
     if self._arrays is None:
       return True
@@ -424,7 +447,7 @@ class ArrayImpl(basearray.Array):
     return any(buf.is_deleted() for buf in self._arrays)
 
   def _check_if_deleted(self):
-    if self._arrays is None:
+    if self.is_deleted():
       raise RuntimeError("Array has been deleted.")
 
   @pxla.use_cpp_method
@@ -573,6 +596,7 @@ def _array_rest_shard_arg(x: ArrayImpl, devices, indices, mode):
 
 
 def _array_shard_arg(x, devices, indices, mode):
+  x._check_if_deleted()
   if mode == pxla.InputsHandlerMode.pmap:
     return _array_pmap_shard_arg(x, devices, indices, mode)
   else:
