@@ -18,19 +18,20 @@
 import contextlib
 import functools
 import itertools
+import logging
 import os
 import sys
 import threading
 from typing import Any, List, Callable, Hashable, NamedTuple, Iterator, Optional
-import warnings
-
-from absl import logging
 
 from jax._src import lib
 from jax._src.lib import jax_jit
 from jax._src.lib import transfer_guard_lib
 from jax._src.lib import xla_client
 
+logger = logging.getLogger(__name__)
+
+
 def bool_env(varname: str, default: bool) -> bool:
   """Read an environment variable and interpret it as a boolean.
 
@@ -125,6 +126,10 @@ class Config:
     update_hook = kwargs.pop("update_hook", None)
     self.add_option(name, default, int, args, kwargs, update_hook=update_hook)
 
+  def DEFINE_float(self, name, default, *args, **kwargs):
+    update_hook = kwargs.pop("update_hook", None)
+    self.add_option(name, default, float, args, kwargs, update_hook=update_hook)
+
   def DEFINE_string(self, name, default, *args, **kwargs):
     update_hook = kwargs.pop("update_hook", None)
     self.add_option(name, default, str, args, kwargs, update_hook=update_hook)
@@ -143,6 +148,7 @@ class Config:
     self.absl_flags = absl_flags
     absl_defs = { bool: absl_flags.DEFINE_bool,
                   int:  absl_flags.DEFINE_integer,
+                  float: absl_flags.DEFINE_float,
                   str:  absl_flags.DEFINE_string,
                   'enum': absl_flags.DEFINE_enum }
 
@@ -285,6 +291,88 @@ class Config:
 
     return _StateContextManager(name, help, update_thread_local_hook, validate)
 
+  def define_int_state(
+      self, name: str, default: Optional[int],
+      help: str, update_global_hook: Optional[Callable[[str], None]] = None,
+      update_thread_local_hook: Optional[Callable[[Optional[str]], None]] \
+        = None):
+    """Set up thread-local state and return a contextmanager for managing it.
+    Args:
+      name: string, converted to lowercase to define the name of the config
+        option (and absl flag). It is converted to uppercase to define the
+        corresponding shell environment variable.
+      enum_values: list of strings representing the possible values for the
+        option.
+      default: optional int, default value.
+      help: string, used to populate the flag help information as well as the
+        docstring of the returned context manager.
+    Returns:
+      A contextmanager to control the thread-local state value.
+    See docstring for ``define_bool_state``.
+    """
+    name = name.lower()
+    default_env = os.getenv(name.upper(), default)
+    if default_env is not None:
+      try:
+        default = int(default_env)
+      except ValueError:
+        raise ValueError(f"Invalid value \"{default_env}\" for JAX flag {name}")
+    self.DEFINE_integer(name, default, help=help, update_hook=update_global_hook)
+    self._contextmanager_flags.add(name)
+
+    def get_state(self):
+      val = getattr(_thread_local_state, name, unset)
+      return val if val is not unset else self._read(name)
+    setattr(Config, name, property(get_state))
+
+    def validate(new_val):
+      if new_val is not None and not isinstance(new_val, int):
+        raise ValueError(f'new int config value must be None or of type int, '
+                         f'got {new_val} of type {type(new_val)}')
+
+    return _StateContextManager(name, help, update_thread_local_hook, validate)
+
+  def define_float_state(
+      self, name: str, default: Optional[float],
+      help: str, update_global_hook: Optional[Callable[[str], None]] = None,
+      update_thread_local_hook: Optional[Callable[[Optional[str]], None]] \
+        = None):
+    """Set up thread-local state and return a contextmanager for managing it.
+    Args:
+      name: string, converted to lowercase to define the name of the config
+        option (and absl flag). It is converted to uppercase to define the
+        corresponding shell environment variable.
+      enum_values: list of strings representing the possible values for the
+        option.
+      default: optional float, default value.
+      help: string, used to populate the flag help information as well as the
+        docstring of the returned context manager.
+    Returns:
+      A contextmanager to control the thread-local state value.
+    See docstring for ``define_bool_state``.
+    """
+    name = name.lower()
+    default_env = os.getenv(name.upper(), default)
+    if default_env is not None:
+      try:
+        default = float(default_env)
+      except ValueError:
+        raise ValueError(f"Invalid value \"{default_env}\" for JAX flag {name}")
+    self.DEFINE_float(name, default, help=help, update_hook=update_global_hook)
+    self._contextmanager_flags.add(name)
+
+    def get_state(self):
+      val = getattr(_thread_local_state, name, unset)
+      return val if val is not unset else self._read(name)
+    setattr(Config, name, property(get_state))
+
+    def validate(new_val):
+      if new_val is not None and not isinstance(new_val, (float, int)):
+        raise ValueError(f'new float config value must be None or of type float, '
+                         f'got {new_val} of type {type(new_val)}')
+
+    return _StateContextManager(name, help, update_thread_local_hook, validate)
+
   def define_string_state(
       self, name: str, default: Optional[str], help: str,
       update_global_hook: Optional[Callable[[str], None]] = None,
@@ -643,7 +731,7 @@ log_compiles = config.define_bool_state(
     name='jax_log_compiles',
     default=False,
     help=('Log a message each time every time `jit` or `pmap` compiles an XLA '
-          'computation. Logging is performed with `absl.logging`. When this '
+          'computation. Logging is performed with `logging`. When this '
           'option is set, the log level is WARNING; otherwise the level is '
           'DEBUG.'))
 
@@ -653,12 +741,10 @@ parallel_functions_output_gda = config.define_bool_state(
     help='If True, pjit will output GDAs.')
 
 def _update_jax_array_global(val):
-  if lib.xla_extension_version >= 92:
-    lib.jax_jit.global_state().jax_array = val
+  lib.jax_jit.global_state().jax_array = val
 
 def _update_jax_array_thread_local(val):
-  if lib.xla_extension_version >= 92:
-    lib.jax_jit.thread_local_state().jax_array = val
+  lib.jax_jit.thread_local_state().jax_array = val
 
 jax_array = config.define_bool_state(
     name='jax_array',
@@ -670,11 +756,28 @@ jax_array = config.define_bool_state(
           'used.'))
 
 
+spmd_mode = config.define_enum_state(
+    name='jax_spmd_mode',
+    enum_values=['allow_all', 'allow_jit', 'allow_pjit'],
+    # TODO(yashkatariya): Default to `allow_jit` when the training wheels come
+    # off.
+    default='allow_pjit',
+    help=("Decides whether Math on `jax.Array`'s that are not fully addressable "
+          "(i.e. spans across multiple processes) is allowed. The options are: "
+          "* allow_pjit: Default, only `pjit` computations are allowed to "
+          "    execute on non-fully addressable `jax.Array`s\n"
+          "* allow_jit: `pjit` and `jax.jit` computations are allowed to "
+          "    execute on non-fully addressable `jax.Array`s\n"
+          "* allow_all: `jnp`, normal math (like `a + b`, etc), `pjit`, "
+          "     `jax.jit` and all other operations are allowed to "
+          "     execute on non-fully addresable `jax.Array`s."))
+
+
 distributed_debug = config.define_bool_state(
     name='jax_distributed_debug',
     default=False,
     help=('Enable logging useful for debugging multi-process distributed '
-          'computations. Logging is performed with `absl.logging` at WARNING '
+          'computations. Logging is performed with `logging` at WARNING '
           'level.'))
 
 
@@ -692,6 +795,18 @@ default_prng_impl = config.define_enum_state(
     help=('Select the default PRNG implementation, used when one is not '
           'explicitly provided at seeding time.'))
 
+threefry_partitionable = config.define_bool_state(
+    name='jax_threefry_partitionable',
+    default=False,
+    upgrade=True,
+    help=('Enables internal threefry PRNG implementation changes that '
+          'render it automatically partitionable in some cases. For use '
+          'with pjit and/or jax_array=True. Without this flag, using the '
+          'standard jax.random pseudo-random number generation may result '
+          'in extraneous communication and/or redundant distributed '
+          'computation. With this flag, the communication overheads disappear '
+          'in some cases.'))
+
 enable_custom_vjp_by_custom_transpose = config.define_bool_state(
     name='jax_enable_custom_vjp_by_custom_transpose',
     default=False,
@@ -709,6 +824,13 @@ raise_persistent_cache_errors = config.define_bool_state(
           'continue. Defaults to false so cache bugs or intermittent issues '
           'are non-fatal.'))
 
+persistent_cache_min_compile_time_secs = config.define_float_state(
+    name='jax_persistent_cache_min_compile_time_secs',
+    default=1,
+    help=('The minimum compile time of a computation to be written to the '
+          'persistent compilation cache. This threshold can be raised to '
+          'decrease the number of entries written to the cache.'))
+
 hlo_source_file_canonicalization_regex = config.define_string_state(
     name='jax_hlo_source_file_canonicalization_regex',
     default=None,
@@ -772,7 +894,7 @@ def _validate_default_device(val):
     # TODO(skyewm): this is a workaround for non-PJRT Device types. Remove when
     # all JAX backends use a single C++ device interface.
     if 'Device' in str(type(val)):
-      logging.info(
+      logger.info(
           'Allowing non-`xla_client.Device` default device: %s, type: %s',
           repr(val), type(val))
       return
@@ -894,7 +1016,7 @@ config.define_bool_state(
 # TODO(b/205307544): Remove flag once coordination service has rolled out.
 config.define_bool_state(
     name='jax_coordination_service',
-    default=(lib.xla_extension_version >= 80),
+    default=True,
     help=(
          'Use coordination service (experimental) instead of the default PjRT '
          'distributed runtime.'
