@@ -1,4 +1,4 @@
-# Copyright 2021 Google LLC
+# Copyright 2021 The JAX Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -115,7 +115,7 @@ def maybe_bdim_at_front(x, bdim):
 def vmap_unrestricted(f: lu.WrappedFun, *args, in_axes, axis_name, axis_size):
   f, out_axes = batching.batch_subtrace(f)
   f = batching._batch_outer(f, axis_name, axis_size, in_axes,
-                            batching.BatchTrace)
+                            batching.BatchTrace, None)
   outs = f.call_wrapped(*args)
   return outs, out_axes()
 
