@@ -16,21 +16,20 @@
 from __future__ import annotations
 
 import atexit
-import collections
 import contextlib
 from functools import partial
 import itertools
 import time
 from typing import (
-    Any, Callable, Dict, Optional, Sequence, Set, Tuple, List, Type, Union,
-    TYPE_CHECKING, Iterator)
+    Any, Callable, Dict, Iterable, Iterator, Optional, Sequence,
+    Set, Tuple, List, Type, Union)
 from typing_extensions import Protocol
+import logging
 import os
 import re
 import threading
 import warnings
 
-from absl import logging
 import numpy as np
 
 import jax
@@ -55,7 +54,7 @@ from jax._src.lib import xla_bridge as xb
 from jax._src.lib import xla_client as xc
 import jax._src.util as util
 from jax._src.util import flatten, unflatten
-from etils import epath
+from jax._src import path
 
 
 FLAGS = flags.FLAGS
@@ -83,6 +82,8 @@ CompileOptions = xc.CompileOptions
 map, unsafe_map = util.safe_map, map
 zip, unsafe_zip = util.safe_zip, zip
 
+logger = logging.getLogger(__name__)
+
 # This flag is set on exit; no logging should be attempted
 _on_exit = False
 
@@ -221,18 +222,34 @@ def _device_from_arg_devices(devices: Sequence[Optional[Device]]) -> Optional[De
 
 # JIT execution
 
-def _xla_call_impl(fun: lu.WrappedFun, *args, device, backend, name,
-                   donated_invars, inline, keep_unused: bool):
+
+def _xla_call_impl_lazy(fun: lu.WrappedFun, *args, device, backend, name,
+                        donated_invars, inline, keep_unused: bool):
   del inline  # Only used at tracing time
   if fun.in_type is None:
-    arg_specs = unsafe_map(arg_spec, args)
+    arg_specs: Iterable[Any] = unsafe_map(arg_spec, args)
   else:
     # fun.in_type is used for dynamic shapes.
     if config.jax_array:
       raise NotImplementedError('Dynamic shapes do not work with Array.')
     arg_specs = [(None, getattr(x, '_device', None)) for x in args]
-  compiled_fun = xla_callable(fun, device, backend, name, donated_invars,
-                              keep_unused, *arg_specs)
+  return xla_callable(fun, device, backend, name, donated_invars, keep_unused,
+                      *arg_specs)
+
+
+def _xla_call_impl(fun: lu.WrappedFun, *args, device, backend, name,
+                   donated_invars, inline, keep_unused: bool):
+  compiled_fun = _xla_call_impl_lazy(
+      fun,
+      *args,
+      device=device,
+      backend=backend,
+      name=name,
+      donated_invars=donated_invars,
+      inline=inline,
+      keep_unused=keep_unused)
+  # TODO(parkers): Maybe apply this more generally in the case of the c++
+  # fallback.
   try:
     return compiled_fun(*args)
   except FloatingPointError:
@@ -361,7 +378,7 @@ def log_elapsed_time(fmt: str):
     start_time = time.time()
     yield
     elapsed_time = time.time() - start_time
-    logging.log(log_priority, fmt.format(elapsed_time=elapsed_time))
+    logger.log(log_priority, fmt.format(elapsed_time=elapsed_time))
 
 
 def should_tuple_args(num_args: int, platform: str):
@@ -421,7 +438,7 @@ def lower_xla_callable(
     fun = lu.annotate(fun, in_type)
   else:
     assert abstract_args == (None,) * len(abstract_args)
-    abstract_args = [aval for aval, _ in fun.in_type]
+    abstract_args = tuple(aval for aval, _ in fun.in_type)
 
   with log_elapsed_time(f"Finished tracing + transforming {fun.__name__} "
                         "for jit in {elapsed_time} sec"):
@@ -477,7 +494,7 @@ def lower_xla_callable(
       msg = f"Compiling {fun.__name__} ({id(fun)}) for {len(abstract_args)} args."
     else:
       msg = f"Compiling {fun.__name__} ({id(fun)} for args {abstract_args}."
-    logging.log(log_priority, msg)
+    logger.log(log_priority, msg)
 
   raise_warnings_or_errors_for_jit_of_pmap(nreps, backend, name, jaxpr)
 
@@ -680,7 +697,7 @@ def _input_handler(backend: Backend,
   assert config.jax_dynamic_shapes
 
   # Precompute how to grab implicit inputs from explicit inputs' axis sizes.
-  which_explicit = which_explicit or [True] * len(in_avals)
+  which_explicit = which_explicit or (True,) * len(in_avals)
   implicit_idxs = {i for i, ex in enumerate(which_explicit) if not ex}
   implicit_args_from_axes: List[Tuple[int, int, int]] = []
   for arg_idx, aval in enumerate(in_avals):
@@ -724,7 +741,7 @@ def _input_handler(backend: Backend,
 
 def _result_handler(backend: Backend,
                     sticky_device: Optional[Device],
-                    out_type: Optional[pe.OutputType]
+                    out_type: pe.OutputType,
                     ) -> Callable:
   out_avals, kept_outputs = util.unzip2(out_type)
   handlers = map(partial(aval_to_result_handler, sticky_device), out_avals)
@@ -975,6 +992,7 @@ class XlaComputation(stages.XlaLowering):
         self._executable = XlaCompiledComputation.from_trivial_jaxpr(
             **self.compile_args)
       else:
+        assert self._out_type is not None
         self._executable = XlaCompiledComputation.from_xla_computation(
             self.name, self._hlo, self._in_type, self._out_type,
             **self.compile_args)
@@ -1004,7 +1022,7 @@ def _make_string_safe_for_filename(s: str) -> str:
 def _dump_ir_to_file(name: str, ir: str):
   id = next(_ir_dump_counter)
   name = f"jax_ir{id}_{_make_string_safe_for_filename(name)}.mlir"
-  name = epath.Path(FLAGS.jax_dump_ir_to) / name
+  name = path.Path(FLAGS.jax_dump_ir_to) / name
   name.write_text(ir)
 
 
@@ -1038,17 +1056,22 @@ def compile_or_get_cached(backend, computation: ir.Module, compile_options,
   # TODO(b/232263664): Remove check when JitRt is enabled by default.
   if "--xla_gpu_enable_xla_runtime_executable=true" in os.environ.get("XLA_FLAGS", ""):
     supported_platforms.append("gpu")
+  # (b/233850967) CPU caching can be enabled if XLA Runtime is enabled.
+  if "--xla_cpu_use_xla_runtime=true" in os.environ.get("XLA_FLAGS", ""):
+    supported_platforms.append("cpu")
   if cc.is_initialized() and backend.platform in supported_platforms:
     cached_executable = _cache_read(serialized_computation, module_name,
                                     compile_options, backend)
     if cached_executable is not None:
-      logging.info("Persistent compilation cache hit for '%s'", module_name)
+      logger.info("Persistent compilation cache hit for '%s'", module_name)
       return cached_executable
     else:
+      start_time = time.monotonic()
       compiled = backend_compile(backend, serialized_computation,
                                  compile_options, host_callbacks)
-      _cache_write(serialized_computation, module_name,  compile_options,
-                   backend, compiled)
+      compile_time = time.monotonic() - start_time
+      _cache_write(serialized_computation, compile_time, module_name,
+                   compile_options, backend, compiled)
       return compiled
 
   return backend_compile(backend, serialized_computation, compile_options,
@@ -1073,16 +1096,31 @@ def _cache_read(computation: Union[str, bytes, ir.Module], module_name: str,
     return None
 
 
-def _cache_write(computation: Union[str, bytes, ir.Module], module_name: str,
-                 compile_options: CompileOptions, backend: Backend,
-                 compiled: XlaLoadedExecutable):
-  """Writes `computation` to the persistent compilation cache."""
+def _cache_write(serialized_computation: Union[str, bytes, ir.Module],
+                 compile_time_secs: float,
+                 module_name: str, compile_options: CompileOptions,
+                 backend: Backend, compiled: XlaLoadedExecutable):
+  """Writes `serialized_computation` to the persistent compilation cache."""
   # Avoid import cycle between jax and jax.experimental
   from jax.experimental.compilation_cache import compilation_cache as cc
 
+  min_compile_time = config.jax_persistent_cache_min_compile_time_secs
+  if min_compile_time:
+    if compile_time_secs < min_compile_time:
+      logging.info(
+          "Not writing persistent cache entry for '%s' because it took < %.2f "
+          "seconds to compile (%.2fs)", module_name, min_compile_time,
+          compile_time_secs)
+      return
+    else:
+      logging.info(
+          "'%s' took at least %.2f seconds to compile (%.2fs), writing "
+          "persistent cache entry", module_name, min_compile_time,
+          compile_time_secs)
+
   try:
-    cc.put_executable(module_name, computation, compile_options, compiled,
-                      backend)
+    cc.put_executable(module_name, serialized_computation, compile_options,
+                      compiled, backend)
   except Exception as ex:
     if config.jax_raise_persistent_cache_errors:
       raise
@@ -1090,6 +1128,26 @@ def _cache_write(computation: Union[str, bytes, ir.Module], module_name: str,
         f"Error writing persistent compilation cache entry for "
         f"'{module_name}': {type(ex).__name__}: {ex}")
 
+
+def _instruction_count(module: ir.Module, max_count: Optional[int] = None):
+
+  def _blocks_count(blocks, count):
+    for block in blocks:
+      for op in block.operations:
+        count += 1
+        # Untested premature performance optimization
+        if max_count is not None and count >= max_count:
+          return max_count
+        for region in op.regions:
+          count = _blocks_count(region.blocks, count)
+    return count
+
+  count = 0
+  for func in module.body.operations:
+    count = _blocks_count(func.body.blocks, count)
+  return count
+
+
 def get_buffer_counts(out_avals, ordered_effects, has_unordered_effects):
   buffer_counts = [aval_to_num_buffers(aval) for aval in out_avals]
   if ordered_effects or has_unordered_effects:
@@ -1115,7 +1173,7 @@ class XlaCompiledComputation(stages.XlaExecutable):
   @staticmethod
   def from_xla_computation(name: str, xla_computation: Optional[ir.Module],
                            in_type: Optional[pe.InputType],
-                           out_type: Optional[pe.OutputType], nreps: int,
+                           out_type: pe.OutputType, nreps: int,
                            device: Optional[Device], backend: Backend,
                            tuple_args: bool,
                            in_avals: Sequence[core.AbstractValue],
@@ -1299,6 +1357,12 @@ def _device_put_impl(
         f"Argument '{x}' of type {type(x)} is not a valid JAX type") from err
 
   if isinstance(device, sharding.Sharding):
+    if not jax.config.jax_array:
+      raise RuntimeError(
+          "Please enable `jax_array` to use device_put with a `Sharding`. "
+          "You can use jax.config.update('jax_array', True) or set JAX_ARRAY=1 "
+          "environment variable or set the `jax_array` boolean flag to "
+          "something true-like.")
     s = device
     if not s.is_fully_addressable:  # type: ignore
       raise ValueError(
