@@ -21,13 +21,15 @@
 
 
 import functools
-from typing import Any, Dict, List
+from typing import cast, overload, Any, Dict, List, Optional, Set, Tuple, Union
+from typing_extensions import Literal
 
 import numpy as np
 
 import jax
 from jax._src.config import flags, config
 from jax._src.lib import xla_client
+from jax._src.typing import DType, DTypeLike, OpaqueDType
 
 from jax._src import traceback_util
 traceback_util.register_exclusion(__file__)
@@ -44,12 +46,12 @@ int_: type = np.int32 if config.jax_default_dtype_bits == '32' else np.int64
 uint: type = np.uint32 if config.jax_default_dtype_bits == '32' else np.uint64
 float_: type = np.float32 if config.jax_default_dtype_bits == '32' else np.float64
 complex_: type = np.complex64 if config.jax_default_dtype_bits == '32' else np.complex128
-_default_types = {'b': bool_, 'i': int_, 'u': uint, 'f': float_, 'c': complex_}
+_default_types: Dict[str, type] = {'b': bool_, 'i': int_, 'u': uint, 'f': float_, 'c': complex_}
 
 # Trivial vectorspace datatype needed for tangent values of int/bool primals
 float0: np.dtype = np.dtype([('float0', np.void, 0)])
 
-_dtype_to_32bit_dtype = {
+_dtype_to_32bit_dtype: Dict[DType, DType] = {
     np.dtype('int64'): np.dtype('int32'),
     np.dtype('uint64'): np.dtype('uint32'),
     np.dtype('float64'): np.dtype('float32'),
@@ -59,7 +61,7 @@ _dtype_to_32bit_dtype = {
 # Note: we promote narrow types to float32 here for backward compatibility
 # with earlier approaches. We might consider revisiting this, or perhaps
 # tying the logic more closely to the type promotion lattice.
-_dtype_to_inexact = {
+_dtype_to_inexact: Dict[DType, DType] = {
     np.dtype(k): np.dtype(v) for k, v in [
         ('bool', 'float32'),
         ('uint8', 'float32'), ('int8', 'float32'),
@@ -69,19 +71,19 @@ _dtype_to_inexact = {
     ]
 }
 
-def to_numeric_dtype(dtype):
+def to_numeric_dtype(dtype: DTypeLike) -> DType:
   """Promotes a dtype into an numeric dtype, if it is not already one."""
-  dtype = np.dtype(dtype)
-  return np.dtype('int32') if dtype == np.dtype('bool') else dtype
+  dtype_ = np.dtype(dtype)
+  return np.dtype('int32') if dtype_ == np.dtype('bool') else dtype_
 
 
-def to_inexact_dtype(dtype):
+def to_inexact_dtype(dtype: DTypeLike) -> DType:
   """Promotes a dtype into an inexact dtype, if it is not already one."""
-  dtype = np.dtype(dtype)
-  return _dtype_to_inexact.get(dtype, dtype)
+  dtype_ = np.dtype(dtype)
+  return _dtype_to_inexact.get(dtype_, dtype_)
 
 
-def to_complex_dtype(dtype):
+def to_complex_dtype(dtype: DTypeLike) -> DType:
   ftype = to_inexact_dtype(dtype)
   if ftype in [np.dtype('float64'), np.dtype('complex128')]:
     return np.dtype('complex128')
@@ -89,32 +91,41 @@ def to_complex_dtype(dtype):
 
 
 @functools.lru_cache(maxsize=None)
-def _canonicalize_dtype(x64_enabled, dtype):
+def _canonicalize_dtype(x64_enabled: bool, allow_opaque_dtype: bool, dtype: Any) -> Union[DType, OpaqueDType]:
   """Convert from a dtype to a canonical dtype based on config.x64_enabled."""
   if jax.core.is_opaque_dtype(dtype):
+    if not allow_opaque_dtype:
+      raise ValueError(f"Internal: canonicalize_dtype called onopaque dtype {dtype} "
+                       "with allow_opaque_dtype=False")
     return dtype
   try:
-    dtype = np.dtype(dtype)
+    dtype_ = np.dtype(dtype)
   except TypeError as e:
     raise TypeError(f'dtype {dtype!r} not understood') from e
 
   if x64_enabled:
-    return dtype
+    return dtype_
   else:
-    return _dtype_to_32bit_dtype.get(dtype, dtype)
+    return _dtype_to_32bit_dtype.get(dtype_, dtype_)
 
-def canonicalize_dtype(dtype):
-  return _canonicalize_dtype(config.x64_enabled, dtype)
+@overload
+def canonicalize_dtype(dtype: Any, allow_opaque_dtype: Literal[False] = False) -> DType: ...
+
+@overload
+def canonicalize_dtype(dtype: Any, allow_opaque_dtype: bool = False) -> Union[DType, OpaqueDType]: ...
+
+def canonicalize_dtype(dtype: Any, allow_opaque_dtype: bool = False) -> Union[DType, OpaqueDType]:
+  return _canonicalize_dtype(config.x64_enabled, allow_opaque_dtype, dtype)
 
 # Default dtypes corresponding to Python scalars.
-python_scalar_dtypes : dict = {
+python_scalar_dtypes : Dict[type, DType] = {
   bool: np.dtype('bool'),
   int: np.dtype('int64'),
   float: np.dtype('float64'),
   complex: np.dtype('complex128'),
 }
 
-def scalar_type_of(x):
+def scalar_type_of(x: Any) -> type:
   typ = dtype(x)
   if typ == bfloat16:
     return float
@@ -130,7 +141,7 @@ def scalar_type_of(x):
     raise TypeError(f"Invalid scalar value {x}")
 
 
-def _scalar_type_to_dtype(typ: type, value: Any = None):
+def _scalar_type_to_dtype(typ: type, value: Any = None) -> DType:
   """Return the numpy dtype for the given scalar type.
 
   Raises
@@ -160,7 +171,7 @@ def _scalar_type_to_dtype(typ: type, value: Any = None):
   return dtype
 
 
-def coerce_to_array(x, dtype=None):
+def coerce_to_array(x: Any, dtype: Optional[DTypeLike] = None) -> np.ndarray:
   """Coerces a scalar or NumPy array to an np.array.
 
   Handles Python scalar type promotion according to JAX's rules, not NumPy's
@@ -226,7 +237,7 @@ class finfo(np.finfo):
       return cls._finfo_cache[_bfloat16_dtype]
     return super().__new__(cls, dtype)
 
-def _issubclass(a, b):
+def _issubclass(a: Any, b: Any) -> bool:
   """Determines if ``a`` is a subclass of ``b``.
 
   Similar to issubclass, but returns False instead of an exception if `a` is not
@@ -237,7 +248,7 @@ def _issubclass(a, b):
   except TypeError:
     return False
 
-def issubdtype(a, b) -> bool:
+def issubdtype(a: DTypeLike, b: DTypeLike) -> bool:
   if a == "bfloat16":
     a = bfloat16
   if a == bfloat16:
@@ -259,10 +270,12 @@ def issubdtype(a, b) -> bool:
 can_cast = np.can_cast
 issubsctype = np.issubsctype
 
+JAXType = Union[type, DType]
+
 # Enumeration of all valid JAX types in order.
-_weak_types = [int, float, complex]
-_bool_types: List[np.dtype] = [np.dtype(bool)]
-_int_types: List[np.dtype] = [
+_weak_types: List[JAXType] = [int, float, complex]
+_bool_types: List[JAXType] = [np.dtype(bool)]
+_int_types: List[JAXType] = [
     np.dtype('uint8'),
     np.dtype('uint16'),
     np.dtype('uint32'),
@@ -272,20 +285,20 @@ _int_types: List[np.dtype] = [
     np.dtype('int32'),
     np.dtype('int64'),
 ]
-_float_types: List[np.dtype] = [
+_float_types: List[JAXType] = [
     np.dtype(bfloat16),
     np.dtype('float16'),
     np.dtype('float32'),
     np.dtype('float64'),
 ]
-_complex_types: List[np.dtype] = [
+_complex_types: List[JAXType] = [
     np.dtype('complex64'),
     np.dtype('complex128'),
 ]
 _jax_types = _bool_types + _int_types + _float_types + _complex_types
 _jax_dtype_set = {float0, *_bool_types, *_int_types, *_float_types, *_complex_types}
 
-def _jax_type(dtype, weak_type):
+def _jax_type(dtype: DType, weak_type: bool) -> JAXType:
   """Return the jax type for a dtype and weak type."""
   if weak_type:
     if dtype == bool:
@@ -295,11 +308,11 @@ def _jax_type(dtype, weak_type):
     return type(dtype.type(0).item())
   return dtype
 
-def _dtype_and_weaktype(value):
+def _dtype_and_weaktype(value: Any) -> Tuple[DType, bool]:
   """Return a (dtype, weak_type) tuple for the given input."""
   return dtype(value), any(value is typ for typ in _weak_types) or is_weakly_typed(value)
 
-def _type_promotion_lattice(jax_numpy_dtype_promotion):
+def _type_promotion_lattice(jax_numpy_dtype_promotion: str) -> Dict[JAXType, List[JAXType]]:
   """
   Return the type promotion lattice in the form of a DAG.
   This DAG maps each type to its immediately higher type on the lattice.
@@ -328,7 +341,7 @@ def _type_promotion_lattice(jax_numpy_dtype_promotion):
     raise ValueError(
       f"Unexpected value of jax_numpy_dtype_promotion={jax_numpy_dtype_promotion!r}")
 
-def _make_lattice_upper_bounds(jax_numpy_dtype_promotion):
+def _make_lattice_upper_bounds(jax_numpy_dtype_promotion: str) -> Dict[JAXType, Set[JAXType]]:
   lattice = _type_promotion_lattice(jax_numpy_dtype_promotion)
   upper_bounds = {node: {node} for node in lattice}
   for n in lattice:
@@ -341,7 +354,7 @@ def _make_lattice_upper_bounds(jax_numpy_dtype_promotion):
       upper_bounds[n] |= new_upper_bounds
   return upper_bounds
 
-_lattice_upper_bounds = {
+_lattice_upper_bounds: Dict[str, Dict[JAXType, Set[JAXType]]] = {
   'standard': _make_lattice_upper_bounds('standard'),
   'strict': _make_lattice_upper_bounds('strict'),
 }
@@ -350,7 +363,7 @@ class TypePromotionError(ValueError):
   pass
 
 @functools.lru_cache(512)  # don't use util.memoize because there is no X64 dependence.
-def _least_upper_bound(jax_numpy_dtype_promotion, *nodes):
+def _least_upper_bound(jax_numpy_dtype_promotion: str, *nodes: JAXType) -> JAXType:
   """Compute the least upper bound of a set of nodes.
 
   Args:
@@ -401,7 +414,7 @@ def _least_upper_bound(jax_numpy_dtype_promotion, *nodes):
       "JAX's internal logic; please report it to the JAX maintainers."
     )
 
-def promote_types(a, b):
+def promote_types(a: DTypeLike, b: DTypeLike) -> DType:
   """Returns the type to which a binary operation should cast its arguments.
 
   For details of JAX's type promotion semantics, see :ref:`type-promotion`.
@@ -415,23 +428,23 @@ def promote_types(a, b):
   """
   # Note: we deliberately avoid `if a in _weak_types` here because we want to check
   # object identity, not object equality, due to the behavior of np.dtype.__eq__
-  a = a if any(a is t for t in _weak_types) else np.dtype(a)
-  b = b if any(b is t for t in _weak_types) else np.dtype(b)
-  return np.dtype(_least_upper_bound(config.jax_numpy_dtype_promotion, a, b))
+  a_tp = cast(JAXType, a if any(a is t for t in _weak_types) else np.dtype(a))
+  b_tp = cast(JAXType, b if any(b is t for t in _weak_types) else np.dtype(b))
+  return np.dtype(_least_upper_bound(config.jax_numpy_dtype_promotion, a_tp, b_tp))
 
-def is_weakly_typed(x):
+def is_weakly_typed(x: Any) -> bool:
   try:
     return x.aval.weak_type
   except AttributeError:
     return type(x) in _weak_types
 
-def is_python_scalar(x):
+def is_python_scalar(x: Any) -> bool:
   try:
     return x.aval.weak_type and np.ndim(x) == 0
   except AttributeError:
     return type(x) in python_scalar_dtypes
 
-def dtype(x, *, canonicalize=False):
+def dtype(x: Any, *, canonicalize: bool = False) -> DType:
   """Return the dtype object for a value or type, optionally canonicalized based on X64 mode."""
   if x is None:
     raise ValueError(f"Invalid argument to dtype: {x}.")
@@ -448,7 +461,7 @@ def dtype(x, *, canonicalize=False):
                     "type. Only arrays of numeric types are supported by JAX.")
   return canonicalize_dtype(dt) if canonicalize else dt
 
-def _lattice_result_type(*args):
+def _lattice_result_type(*args: Any) -> Tuple[DType, bool]:
   dtypes, weak_types = zip(*(_dtype_and_weaktype(arg) for arg in args))
   if len(dtypes) == 1:
     out_dtype = dtypes[0]
@@ -469,7 +482,16 @@ def _lattice_result_type(*args):
     out_weak_type = any(result_type is t for t in _weak_types)
   return out_dtype, (out_dtype != bool_) and out_weak_type
 
-def result_type(*args, return_weak_type_flag=False):
+@overload
+def result_type(*args: Any, return_weak_type_flag: Literal[True]) -> Tuple[DType, bool]: ...
+
+@overload
+def result_type(*args: Any, return_weak_type_flag: Literal[False] = False) -> DType: ...
+
+@overload
+def result_type(*args: Any, return_weak_type_flag: bool = False) -> Union[DType, Tuple[DType, bool]]: ...
+
+def result_type(*args: Any, return_weak_type_flag: bool = False) -> Union[DType, Tuple[DType, bool]]:
   """Convenience function to apply JAX argument dtype promotion.
 
   Args:
@@ -483,6 +505,8 @@ def result_type(*args, return_weak_type_flag=False):
     raise ValueError("at least one array or dtype is required")
   dtype, weak_type = _lattice_result_type(*(float_ if arg is None else arg for arg in args))
   if weak_type:
-    dtype = _default_types['f' if dtype == _bfloat16_dtype else dtype.kind]
-  dtype = canonicalize_dtype(dtype)
+    dtype = canonicalize_dtype(
+      _default_types['f' if dtype == _bfloat16_dtype else dtype.kind])
+  else:
+    dtype = canonicalize_dtype(dtype)
   return (dtype, weak_type) if return_weak_type_flag else dtype
