@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Module for the `for_loop` primitive."""
-from functools import partial
+import functools
 import operator
 
 from typing import Any, Callable, Generic, List, Optional, Sequence, Set, Tuple, TypeVar, Union
@@ -35,7 +35,6 @@ from jax._src import state
 from jax._src.util import (partition_list, merge_lists, safe_map, safe_zip,
                            split_list, split_dict)
 import jax.numpy as jnp
-import numpy as np
 
 from jax._src.lax.control_flow import loops
 from jax._src.lax.control_flow.common import _abstractify, _initial_style_jaxpr
@@ -153,7 +152,8 @@ def for_loop(nsteps: Union[int, Sequence[int]],
     outer_step, *rest_steps = nsteps
     def wrapped_body(i, refs):
       vals = tree_map(lambda ref: state.ref_get(ref, ()), refs)
-      vals = for_loop(rest_steps, partial(body, i), vals, unroll=unroll)
+      vals = for_loop(
+          rest_steps, functools.partial(body, i), vals, unroll=unroll)
       tree_map(lambda ref, val: state.ref_set(ref, (), val), refs, vals)
     return for_loop(outer_step, wrapped_body, init_state, unroll=unroll)
   nsteps, = nsteps
@@ -244,16 +244,11 @@ def scan(f: Callable[[Carry, X], Tuple[Carry, Y]],
                          unroll=unroll)
   return init, ys
 
-def _get_ref_state_effects(jaxpr: core.Jaxpr) -> List[Set[StateEffect]]:
-  all_effects = jaxpr.effects
-  return [{eff for eff in all_effects
-           if isinstance(eff, (ReadEffect, WriteEffect, AccumEffect))
-           and eff.ref_aval is v.aval} for v in jaxpr.invars]
-
 @for_p.def_effectful_abstract_eval
 def _for_abstract_eval(*avals, jaxpr, **__):
   # Find out for each of the `Ref`s in our jaxpr what effects they have.
-  jaxpr_aval_effects = _get_ref_state_effects(jaxpr)[1:]
+  jaxpr_aval_effects = state.get_ref_state_effects(
+      [v.aval for v in jaxpr.invars], jaxpr.effects)[1:]
   aval_effects = [set(eff.replace(ref_aval=aval) for eff in effs) for aval, effs
                   in zip(avals, jaxpr_aval_effects)
                   if isinstance(aval, ShapedArrayRef)]
@@ -261,7 +256,7 @@ def _for_abstract_eval(*avals, jaxpr, **__):
   return list(avals), nonlocal_state_effects
 
 @state.register_discharge_rule(for_p)
-def _for_discharge_rule(in_avals, *args: Any, jaxpr: core.Jaxpr,
+def _for_discharge_rule(in_avals, _, *args: Any, jaxpr: core.Jaxpr,
                         reverse: bool, which_linear: Sequence[bool],
                         nsteps: int, unroll: int
                         ) -> Tuple[Sequence[Optional[Any]], Sequence[Any]]:
@@ -303,7 +298,7 @@ def _for_impl_unrolled(body, nsteps, unroll, *args):
   return state
 
 mlir.register_lowering(for_p, mlir.lower_fun(_for_impl, multiple_results=True))
-for_p.def_impl(partial(xla.apply_primitive, for_p))
+for_p.def_impl(functools.partial(xla.apply_primitive, for_p))
 
 def _for_vmap(axis_size, axis_name, main_type, args, dims, *,
               jaxpr, nsteps, reverse, which_linear, unroll):
@@ -391,7 +386,8 @@ def _is_read_only(ref_effects: Set[StateEffect]) -> bool:
 
 def _loop_invariant_outputs(jaxpr: core.Jaxpr) -> List[bool]:
   # Get effects for each of the jaxpr inputs and remove the loop index.
-  ref_effects = _get_ref_state_effects(jaxpr)[1:]
+  ref_effects = state.get_ref_state_effects(
+      [v.aval for v in jaxpr.invars], jaxpr.effects)[1:]
   # We first assume that *read-only `Ref`s* are loop-invariant. We can safely do
   # this because the only way something can be loop-varying is if we write to it
   # at some point. It's *possible* that read-write `Ref`s are loop-invariant but
@@ -442,7 +438,7 @@ def _for_partial_eval(trace: pe.JaxprTrace, *tracers: pe.JaxprTracer,
   else:
     raise Exception("Invalid fixpoint")
   del out_unknowns  # redundant since it's the same as `in_unknowns`
-  tracers = tuple(trace.instantiate_const(t) if uk else t
+  tracers = tuple(trace.instantiate_const(t) if uk else t  # type: ignore
                   for t, uk in zip(tracers, in_unknowns))
 
   # We use `partial_eval_jaxpr_custom` here because it won't remove effectful
@@ -662,9 +658,10 @@ def _convert_outputs_to_writes(
       else:
         res_ref[i] = res_val
     return []
+  # TODO(mattjj, sharadmv): better handling of tokens, which don't have shape/dtype
   res_ref_avals = [ShapedArrayRef(v.aval.shape, v.aval.dtype)  # pytype: disable=attribute-error
                    if loop_invar else
-                   ShapedArrayRef((nsteps, *v.aval.shape), v.aval.dtype)
+                   ShapedArrayRef((nsteps, *v.aval.shape), v.aval.dtype)  # pytype: disable=attribute-error
                    for v, loop_invar in zip(jaxpr.outvars, loop_invar_res)]
   jaxpr, _, consts = pe.trace_to_jaxpr_dynamic(
       eval_jaxpr, [*in_avals, *res_ref_avals])
@@ -786,3 +783,9 @@ def discharged_for_loop(nsteps, body, init_state, *, reverse: bool = False):
     return out_flat
   out_flat = loops.fori_loop(0, nsteps, fori_body, flat_state)
   return tree_unflatten(state_tree, out_flat)
+
+def run_state(f, init_state):
+  @functools.wraps(f)
+  def wrapped_body(_, *args):
+    return f(*args)
+  return for_loop(1, wrapped_body, init_state)
