@@ -18,23 +18,17 @@ from typing import Union, Sequence
 
 import numpy as np
 
-import jax
 from jax._src.api import jit, linear_transpose, ShapeDtypeStruct
 from jax.core import Primitive
 from jax.interpreters import mlir
 from jax.interpreters import xla
 from jax._src.util import prod
-from jax._src import dtypes
 from jax import lax
 from jax.interpreters import ad
 from jax.interpreters import batching
 from jax._src.lib.mlir.dialects import mhlo
-from jax._src.lib import mlir_api_version
 from jax._src.lib import xla_client
-# TODO(phawkins): remove pocketfft references when the minimum jaxlib version
-# is 0.3.17 or newer.
 from jax._src.lib import ducc_fft
-from jax._src.lib import pocketfft
 from jax._src.numpy.util import _promote_dtypes_complex, _promote_dtypes_inexact
 
 __all__ = [
@@ -110,29 +104,16 @@ def fft_abstract_eval(x, fft_type, fft_lengths):
 
 def _fft_lowering(ctx, x, *, fft_type, fft_lengths):
   out_aval, = ctx.avals_out
-  if mlir_api_version < 31:
-    return [
-        mhlo.FftOp(
-            mlir.aval_to_ir_type(out_aval), x,
-            mhlo.FftTypeAttr.get(fft_type.name),
-            mlir.dense_int_elements(fft_lengths)).result
-    ]
-  else:
-    return [
-        mhlo.FftOp(x, mhlo.FftTypeAttr.get(fft_type.name),
-                   mlir.dense_int_elements(fft_lengths)).result
-    ]
+  return [
+      mhlo.FftOp(x, mhlo.FftTypeAttr.get(fft_type.name),
+                  mlir.dense_int_elements(fft_lengths)).result
+  ]
 
 
 def _fft_lowering_cpu(ctx, x, *, fft_type, fft_lengths):
   x_aval, = ctx.avals_in
-  if ducc_fft:
-    return [ducc_fft.ducc_fft_mhlo(x, x_aval.dtype, fft_type=fft_type,
-                                   fft_lengths=fft_lengths)]
-  else:
-    return [pocketfft.pocketfft_mhlo(x, x_aval.dtype, fft_type=fft_type,
-                                    fft_lengths=fft_lengths)]
-
+  return [ducc_fft.ducc_fft_mhlo(x, x_aval.dtype, fft_type=fft_type,
+                                  fft_lengths=fft_lengths)]
 
 def _naive_rfft(x, fft_lengths):
   y = fft(x, xla_client.FftType.FFT, fft_lengths)
