@@ -41,6 +41,7 @@ from jax._src.lax import eigh as lax_eigh
 from jax._src.lax import lax as lax_internal
 from jax._src.lax import svd as lax_svd
 from jax._src.lib import lapack
+from jax._src.lib import mlir_api_version
 
 from jax._src.lib import gpu_linalg
 from jax._src.lib import gpu_solver
@@ -379,12 +380,6 @@ def _T(x: Array) -> Array: return jnp.swapaxes(x, -1, -2)
 def _H(x: Array) -> Array: return jnp.conj(_T(x))
 def symmetrize(x: Array) -> Array: return (x + _H(x)) / 2
 
-def _unpack_tuple(f: TFun, n: int) -> TFun:
-  def g(c, *args, **kwargs):
-    t = f(c, *args, **kwargs)
-    return (xops.GetTupleElement(t, i) for i in range(n))
-  return cast(TFun, g)
-
 # primitives
 
 _cpu_lapack_types = {np.dtype(np.float32), np.dtype(np.float64),
@@ -879,10 +874,16 @@ def _triangular_solve_lowering(
     transpose = "NO_TRANSPOSE"
   else:
     transpose = "ADJOINT" if conjugate_a else "TRANSPOSE"
-  return mhlo.TriangularSolveOp(
-      mlir.aval_to_ir_type(out_aval), a, b, ir.BoolAttr.get(left_side),
-      ir.BoolAttr.get(lower), ir.BoolAttr.get(unit_diagonal),
-      mhlo.TransposeAttr.get(transpose)).results
+  if mlir_api_version < 36:
+    return mhlo.TriangularSolveOp(
+        mlir.aval_to_ir_type(out_aval), a, b, ir.BoolAttr.get(left_side),
+        ir.BoolAttr.get(lower), ir.BoolAttr.get(unit_diagonal),
+        mhlo.TransposeAttr.get(transpose)).results
+  else:
+    return mhlo.TriangularSolveOp(
+        a, b, ir.BoolAttr.get(left_side),
+        ir.BoolAttr.get(lower), ir.BoolAttr.get(unit_diagonal),
+        mhlo.TransposeAttr.get(transpose)).results
 
 mlir.register_lowering(triangular_solve_p, _triangular_solve_lowering)
 
@@ -906,48 +907,21 @@ def _triangular_solve_cpu_lower(
       transpose = "ADJOINT" if conjugate_a else "TRANSPOSE"
     else:
       transpose = "NO_TRANSPOSE"
-    return mhlo.TriangularSolveOp(b.type, a, b, ir.BoolAttr.get(left_side),
-                                  ir.BoolAttr.get(lower),
-                                  ir.BoolAttr.get(unit_diagonal),
-                                  mhlo.TransposeAttr.get(transpose)).results
+    if mlir_api_version < 36:
+      return mhlo.TriangularSolveOp(b.type, a, b, ir.BoolAttr.get(left_side),
+                                    ir.BoolAttr.get(lower),
+                                    ir.BoolAttr.get(unit_diagonal),
+                                    mhlo.TransposeAttr.get(transpose)).results
+    else:
+      return mhlo.TriangularSolveOp(a, b, ir.BoolAttr.get(left_side),
+                                    ir.BoolAttr.get(lower),
+                                    ir.BoolAttr.get(unit_diagonal),
+                                    mhlo.TransposeAttr.get(transpose)).results
 
 mlir.register_lowering(triangular_solve_p, _triangular_solve_cpu_lower,
                        platform='cpu')
 
 
-def _triangular_solve_gpu_lower(
-    trsm_impl, ctx, a, b, *, left_side, lower, transpose_a, conjugate_a,
-    unit_diagonal):
-  a_aval, _ = ctx.avals_in
-  m, n = a_aval.shape[-2:]
-  batch = prod(a_aval.shape[:-2])
-  if conjugate_a and not transpose_a:
-    a = chlo.ConjOp(a).result
-    conjugate_a = False
-  if batch > 1 and m <= 256 and n <= 256:
-    return [trsm_impl(a_aval.dtype, a, b, left_side, lower, transpose_a,
-                      conjugate_a, unit_diagonal)]
-  else:
-    # Use the XLA implementation for unbatched triangular_solve.
-    if transpose_a:
-      transpose = "ADJOINT" if conjugate_a else "TRANSPOSE"
-    else:
-      transpose = "NO_TRANSPOSE"
-    return mhlo.TriangularSolveOp(b.type, a, b, ir.BoolAttr.get(left_side),
-                                  ir.BoolAttr.get(lower),
-                                  ir.BoolAttr.get(unit_diagonal),
-                                  mhlo.TransposeAttr.get(transpose)).results
-
-mlir.register_lowering(
-    triangular_solve_p,
-    partial(_triangular_solve_gpu_lower, gpu_solver.cuda_trsm),
-    platform='cuda')
-mlir.register_lowering(
-    triangular_solve_p,
-    partial(_triangular_solve_gpu_lower, gpu_solver.rocm_trsm),
-    platform='rocm')
-
-
 # Support operation for LU decomposition: Transformation of the pivots returned
 # by LU decomposition into permutations.
 
@@ -1183,8 +1157,8 @@ def _lu_jvp_rule(primals, tangents):
   lau = triangular_solve(u, la, left_side=False, transpose_a=False,
                          lower=False)
 
-  l_dot = jnp.matmul(l, jnp.tril(lau, -1))
-  u_dot = jnp.matmul(jnp.triu(lau), u)
+  l_dot = jnp.matmul(l, jnp.tril(lau, -1), precision=lax.Precision.HIGHEST)
+  u_dot = jnp.matmul(jnp.triu(lau), u, precision=lax.Precision.HIGHEST)
   lu_dot = l_dot + u_dot
   return (lu, pivots, permutation), (lu_dot, ad_util.Zero.from_value(pivots),
                                      ad_util.Zero.from_value(permutation))
@@ -1468,15 +1442,6 @@ def _qr_impl(operand, *, full_matrices):
   q, r = xla.apply_primitive(qr_p, operand, full_matrices=full_matrices)
   return q, r
 
-def _qr_translation_rule(ctx, avals_in, avals_out, operand, *, full_matrices):
-  operand_aval, = avals_in
-  shape = operand_aval.shape
-  m, n = shape[-2:]
-  if m == 0 or n == 0:
-    return [_eye_like_xla(ctx.builder, avals_out[0]),
-            _zeros_like_xla(ctx.builder, avals_out[1])]
-  return xops.QR(operand, full_matrices)
-
 def _qr_abstract_eval(operand, *, full_matrices):
   if isinstance(operand, ShapedArray):
     if operand.ndim < 2:
