@@ -23,7 +23,6 @@ import jax
 from jax import core
 from jax._src import ad_util
 from jax._src import dtypes
-from jax._src import source_info_util
 from jax.interpreters import ad
 from jax.interpreters import batching
 from jax.interpreters import mlir
@@ -1094,7 +1093,7 @@ def _gather_dimensions_proto(
 def _gather_dtype_rule(operand, indices, *, fill_value, **kwargs):
   if not dtypes.issubdtype(indices.dtype, np.integer):
     raise ValueError("indices must have an integer type")
-  return dtypes.canonicalize_dtype(operand.dtype)
+  return dtypes.canonicalize_dtype(operand.dtype, allow_opaque_dtype=True)
 
 _rank = lambda arr: len(arr.shape)
 
@@ -2138,6 +2137,6 @@ def _dynamic_slice_indices(
     if isinstance(i, (int, np.integer)):
       result.append(i + lax.convert_element_type(d, _dtype(i)) if i < 0 else i)
       continue
-    d = lax.convert_element_type(d, _dtype(i))
-    result.append(lax.select(i < 0, i + d, i))
+    d_arr = lax.convert_element_type(d, _dtype(i))
+    result.append(lax.select(i < 0, i + d_arr, i))
   return result
