@@ -13,18 +13,21 @@
 # limitations under the License.
 
 import abc
+from typing import Any, Iterable, List, Tuple, Union
 
 import jax
 from jax import core
-from jax._src.numpy.lax_numpy import arange, array, concatenate, expand_dims, linspace, meshgrid, stack, transpose
+import jax._src.numpy.lax_numpy as jnp
 from jax._src.numpy.util import _promote_dtypes
+from jax._src.typing import Array, ArrayLike
+
 import numpy as np
 
 
 __all__ = ["c_", "index_exp", "mgrid", "ogrid", "r_", "s_"]
 
 
-def _make_1d_grid_from_slice(s: slice, op_name: str):
+def _make_1d_grid_from_slice(s: slice, op_name: str) -> Array:
   start = core.concrete_or_error(None, s.start,
                                  f"slice start of jnp.{op_name}") or 0
   stop = core.concrete_or_error(None, s.stop,
@@ -32,9 +35,9 @@ def _make_1d_grid_from_slice(s: slice, op_name: str):
   step = core.concrete_or_error(None, s.step,
                                 f"slice step of jnp.{op_name}") or 1
   if np.iscomplex(step):
-    newobj = linspace(start, stop, int(abs(step)))
+    newobj = jnp.linspace(start, stop, int(abs(step)))
   else:
-    newobj = arange(start, stop, step)
+    newobj = jnp.arange(start, stop, step)
 
   return newobj
 
@@ -44,14 +47,14 @@ class _IndexGrid(abc.ABC):
   sparse: bool
   op_name: str
 
-  def __getitem__(self, key):
+  def __getitem__(self, key: Union[slice, Tuple[slice, ...]]) -> Union[Array, List[Array]]:
     if isinstance(key, slice):
       return _make_1d_grid_from_slice(key, op_name=self.op_name)
-    output = (_make_1d_grid_from_slice(k, op_name=self.op_name) for k in key)
+    output: Iterable[Array] = (_make_1d_grid_from_slice(k, op_name=self.op_name) for k in key)
     with jax.numpy_dtype_promotion('standard'):
       output = _promote_dtypes(*output)
-    output = meshgrid(*output, indexing='ij', sparse=self.sparse)
-    return output if self.sparse else stack(output, 0)
+    output_arr = jnp.meshgrid(*output, indexing='ij', sparse=self.sparse)
+    return output_arr if self.sparse else jnp.stack(output_arr, 0)
 
 
 class _Mgrid(_IndexGrid):
@@ -123,6 +126,9 @@ class _Ogrid(_IndexGrid):
 ogrid = _Ogrid()
 
 
+_IndexType = Union[ArrayLike, str, slice]
+
+
 class _AxisConcat(abc.ABC):
   """Concatenates slices, scalars and array-like objects along a given axis."""
   axis: int
@@ -130,28 +136,27 @@ class _AxisConcat(abc.ABC):
   trans1d: int
   op_name: str
 
-  def __getitem__(self, key):
-    if not isinstance(key, tuple):
-      key = (key,)
+  def __getitem__(self, key: Union[_IndexType, Tuple[_IndexType, ...]]) -> Array:
+    key_tup: Tuple[_IndexType, ...] = key if isinstance(key, tuple) else (key,)
 
     params = [self.axis, self.ndmin, self.trans1d, -1]
 
-    if isinstance(key[0], str):
-      # split off the directive
-      directive, *key = key  # pytype: disable=bad-unpacking
+    directive = key_tup[0]
+    if isinstance(directive, str):
+      key_tup = key_tup[1:]
       # check two special cases: matrix directives
       if directive == "r":
         params[-1] = 0
       elif directive == "c":
         params[-1] = 1
       else:
-        vec = directive.split(",")
+        vec: List[Any] = directive.split(",")
         k = len(vec)
         if k < 4:
           vec += params[k:]
         else:
           # ignore everything after the first three comma-separated ints
-          vec = vec[:3] + params[-1]
+          vec = vec[:3] + [params[-1]]
         try:
           params = list(map(int, vec))
         except ValueError as err:
@@ -162,35 +167,37 @@ class _AxisConcat(abc.ABC):
     axis, ndmin, trans1d, matrix = params
 
     output = []
-    for item in key:
+    for item in key_tup:
       if isinstance(item, slice):
         newobj = _make_1d_grid_from_slice(item, op_name=self.op_name)
+        item_ndim = 0
       elif isinstance(item, str):
         raise ValueError("string directive must be placed at the beginning")
       else:
-        newobj = item
+        newobj = jnp.array(item, copy=False)
+        item_ndim = newobj.ndim
 
-      newobj = array(newobj, copy=False, ndmin=ndmin)
+      newobj = jnp.array(newobj, copy=False, ndmin=ndmin)
 
-      if trans1d != -1 and ndmin - np.ndim(item) > 0:
-        shape_obj = list(range(ndmin))
+      if trans1d != -1 and ndmin - item_ndim > 0:
+        shape_obj = tuple(range(ndmin))
         # Calculate number of left shifts, with overflow protection by mod
         num_lshifts = ndmin - abs(ndmin + trans1d + 1) % ndmin
         shape_obj = tuple(shape_obj[num_lshifts:] + shape_obj[:num_lshifts])
 
-        newobj = transpose(newobj, shape_obj)
+        newobj = jnp.transpose(newobj, shape_obj)
 
       output.append(newobj)
 
-    res = concatenate(tuple(output), axis=axis)
+    res = jnp.concatenate(tuple(output), axis=axis)
 
     if matrix != -1 and res.ndim == 1:
       # insert 2nd dim at axis 0 or 1
-      res = expand_dims(res, matrix)
+      res = jnp.expand_dims(res, matrix)
 
     return res
 
-  def __len__(self):
+  def __len__(self) -> int:
     return 0
 
 
