@@ -161,7 +161,7 @@ def _segment_update(name: str,
   segment_ids = jnp.asarray(segment_ids)
   dtype = data.dtype
   if num_segments is None:
-    num_segments = jnp.max(segment_ids) + 1
+    num_segments = np.max(segment_ids) + 1
   num_segments = core.concrete_or_error(int, num_segments, "segment_sum() `num_segments` argument.")
   if num_segments is not None and num_segments < 0:
     raise ValueError("num_segments must be non-negative.")
