@@ -14,7 +14,7 @@
 
 
 import abc
-from functools import partial
+from functools import partial, reduce
 import operator as op
 from typing import Any, Callable, Hashable, Iterator, NamedTuple, Sequence
 
@@ -44,8 +44,6 @@ from jax._src.numpy import lax_numpy
 import jax._src.pretty_printer as pp
 from jax._src.util import canonicalize_axis, prod, safe_map, safe_zip
 from jax._src.lib import gpu_prng
-from jax._src.lib import xla_client as xc
-from jax._src.lib import xla_extension_version
 
 map, unsafe_map = safe_map, map
 zip, unsafe_zip = safe_zip, zip
@@ -297,24 +295,11 @@ class KeyTyRules:
     op_sharding = sharding._to_xla_op_sharding(aval.ndim)
     key_shape = aval.dtype.impl.key_shape
 
-    if xla_extension_version >= 83:
-      new_op_sharding = op_sharding.clone()
-      tad = list(new_op_sharding.tile_assignment_dimensions)
-      tad.extend([1] * len(key_shape))
-      new_op_sharding.tile_assignment_dimensions = tad
-      return new_op_sharding
-    else:
-      # TODO(yashkatariya): This is hacky. Remove this once
-      # minimum_jaxlib_version is bumped to 0.3.19
-      new_op_sharding = xc.OpSharding()
-      new_op_sharding.type = op_sharding.type
-      new_op_sharding.tile_assignment_devices = op_sharding.tile_assignment_devices
-      tad = list(op_sharding.tile_assignment_dimensions)
-      tad.extend([1] * len(key_shape))
-      new_op_sharding.tile_assignment_dimensions = tad
-      new_op_sharding.last_tile_dims = op_sharding.last_tile_dims
-      new_op_sharding.replicate_on_last_tile_dim = op_sharding.replicate_on_last_tile_dim
-      return new_op_sharding
+    new_op_sharding = op_sharding.clone()
+    tad = list(new_op_sharding.tile_assignment_dimensions)
+    tad.extend([1] * len(key_shape))
+    new_op_sharding.tile_assignment_dimensions = tad
+    return new_op_sharding
 
   @staticmethod
   def result_handler(sticky_device, aval):
@@ -995,6 +980,51 @@ mlir.register_lowering(
     platform='rocm')
 
 
+def iota_32x2_shape(shape):
+  if len(shape) == 0:
+    return (jnp.zeros((), np.dtype('uint32')),) * 2
+  return iota_32x2_shape_p.bind(shape=shape)
+
+iota_32x2_shape_p = core.Primitive('iota_32x2_shape')
+iota_32x2_shape_p.multiple_results = True
+iota_32x2_shape_p.def_impl(partial(xla.apply_primitive, iota_32x2_shape_p))
+
+@iota_32x2_shape_p.def_abstract_eval
+def iota_32x2_shape_abstract_eval(*, shape):
+  return (core.ShapedArray(shape, np.dtype('uint32')),) * 2
+
+def iota_32x2_shape_lowering(ctx, *, shape):
+  def _add(x, y):
+    return mlir.mhlo.AddOp(x, y).result
+
+  def _mul(x, y):
+    x_const = mlir.ir_constant(np.array(x, np.dtype('uint64')),
+                               canonicalize_types=False)
+    x_bcast = mlir.mhlo.BroadcastOp(x_const, mlir.dense_int_elements(shape))
+    return mlir.mhlo.MulOp(x_bcast, y).result
+
+  def _sum(xs):
+    return reduce(_add, xs)
+
+  assert len(shape) > 0
+  aval_out, _ = ctx.avals_out
+  aval_u64 = core.ShapedArray(shape, np.dtype('uint64'))
+  iotas = [mlir.mhlo.IotaOp(mlir.aval_to_ir_type(aval_u64),
+                       mlir.i64_attr(dimension)).result
+           for dimension in range(len(shape))]
+  strides = (*map(int, np.cumprod(shape[1:][::-1])[::-1]), 1)
+  counts = _sum(_mul(s, i) for i, s in zip(iotas, strides))  # type: ignore
+  shift = mlir.ir_constant(np.array(32, np.dtype('uint64')),
+                           canonicalize_types=False)
+  shift = mlir.mhlo.BroadcastOp(shift, mlir.dense_int_elements(shape)).result
+  counts_shifted = mlir.mhlo.ShiftRightLogicalOp(counts, shift).result
+  counts_lo = mlir.mhlo.ConvertOp(mlir.aval_to_ir_type(aval_out), counts).result
+  counts_hi = mlir.mhlo.ConvertOp(mlir.aval_to_ir_type(aval_out),
+                                  counts_shifted).result
+  return (counts_hi, counts_lo)
+mlir.register_lowering(iota_32x2_shape_p, iota_32x2_shape_lowering)
+
+
 @partial(jit, inline=True)
 def threefry_2x32(keypair, count):
   """Apply the Threefry 2x32 hash.
@@ -1049,14 +1079,37 @@ def _threefry_fold_in(key, data):
 
 def threefry_random_bits(key: jnp.ndarray, bit_width, shape):
   """Sample uniform random bits of given width and shape using PRNG key."""
-  return _threefry_random_bits(key, bit_width, shape)
-
-@partial(jit, static_argnums=(1, 2), inline=True)
-def _threefry_random_bits(key: jnp.ndarray, bit_width, shape):
   if not _is_threefry_prng_key(key):
     raise TypeError("threefry_random_bits got invalid prng key.")
   if bit_width not in (8, 16, 32, 64):
     raise TypeError("requires 8-, 16-, 32- or 64-bit field width.")
+
+  if (config.jax_threefry_partitionable and
+      not any(core.is_special_dim_size(d) for d in shape)):
+    return _threefry_random_bits_partitionable(key, bit_width, shape)
+  else:
+    return _threefry_random_bits_original(key, bit_width, shape)
+
+def _threefry_random_bits_partitionable(key: jnp.ndarray, bit_width, shape):
+  if all(core.is_constant_dim(d) for d in shape) and prod(shape) > 2 ** 64:
+    raise NotImplementedError('random bits array of size exceeding 2 ** 64')
+
+  k1, k2 = key
+  counts1, counts2 = iota_32x2_shape(shape)
+  bits1, bits2 = threefry2x32_p.bind(k1, k2, counts1, counts2)
+
+  dtype = UINT_DTYPES[bit_width]
+  if bit_width == 64:
+    bits_hi = lax.convert_element_type(bits1, dtype)
+    bits_lo = lax.convert_element_type(bits2, dtype)
+    return lax.shift_left(bits_hi, dtype(32)) | bits_lo
+  elif bit_width == 32:
+    return bits1 ^ bits2
+  else:
+    return lax.convert_element_type(bits1 ^ bits2, dtype)
+
+@partial(jit, static_argnums=(1, 2), inline=True)
+def _threefry_random_bits_original(key: jnp.ndarray, bit_width, shape):
   size = prod(shape)
   # Compute ceil(bit_width * size / 32) in a way that is friendly to shape
   # polymorphism
