@@ -18,10 +18,11 @@ import scipy.stats as osp_stats
 from jax import lax
 from jax._src.numpy.util import _wraps
 from jax._src.numpy.lax_numpy import _promote_args_inexact, where, inf, logical_or
+from jax._src.typing import Array, ArrayLike
 
 
 @_wraps(osp_stats.uniform.logpdf, update_doc=False)
-def logpdf(x, loc=0, scale=1):
+def logpdf(x: ArrayLike, loc: ArrayLike = 0, scale: ArrayLike = 1) -> Array:
   x, loc, scale = _promote_args_inexact("uniform.logpdf", x, loc, scale)
   log_probs = lax.neg(lax.log(scale))
   return where(logical_or(lax.gt(x, lax.add(loc, scale)),
@@ -29,5 +30,5 @@ def logpdf(x, loc=0, scale=1):
                -inf, log_probs)
 
 @_wraps(osp_stats.uniform.pdf, update_doc=False)
-def pdf(x, loc=0, scale=1):
+def pdf(x: ArrayLike, loc: ArrayLike = 0, scale: ArrayLike = 1) -> Array:
   return lax.exp(logpdf(x, loc, scale))
