@@ -11,16 +11,18 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 import abc
 import functools
 from collections import Counter
-from typing import Sequence, Tuple, Optional, Mapping, Dict, Set, Union, cast
+import operator as op
+from typing import (Sequence, List, Tuple, Optional, Mapping, Dict, Set,
+                    FrozenSet, Union, cast)
 
-from jax._src.util import safe_zip
-from jax._src.lib import xla_bridge as xb
+import jax
+from jax._src.util import safe_map, safe_zip
 from jax._src.lib import xla_client as xc
-from jax._src.lib import xla_extension_version
 from jax.interpreters import pxla, mlir
 
 import numpy as np
@@ -59,8 +61,8 @@ class Sharding(metaclass=abc.ABCMeta):
   @pxla.maybe_cached_property
   def addressable_devices(self) -> Set[Device]:
     """A set of addressable devices by the current process"""
-    process_index = xb.process_index()
-    return {d for d in self.device_set if d.process_index == process_index}
+    return {d for d in self.device_set
+            if d.process_index == d.client.process_index()}
 
   @pxla.maybe_cached_property
   def is_fully_addressable(self) -> bool:
@@ -74,9 +76,8 @@ class Sharding(metaclass=abc.ABCMeta):
   @functools.lru_cache(maxsize=4096)
   def addressable_devices_indices_map(
       self, global_shape: Shape) -> Mapping[Device, Optional[Index]]:
-    process_index = xb.process_index()
     return {d: ind for d, ind in self.devices_indices_map(global_shape).items()
-            if d.process_index == process_index}
+            if d.process_index == d.client.process_index()}
 
 
 @pxla.use_cpp_class(xc.XLACompatibleSharding if xc._version >= 94 else None)
@@ -95,10 +96,17 @@ class XLACompatibleSharding(Sharding, metaclass=abc.ABCMeta):
   #############################################################################
   # Default implementations below that all subclasses will inherit.
 
+  @functools.lru_cache(maxsize=4096)
+  def devices_indices_map(self, global_shape: Shape) -> Mapping[Device, Index]:
+    op_sharding = self._to_xla_op_sharding(len(global_shape))
+    op_sharding_sharding = OpShardingSharding(self._device_assignment,
+                                              op_sharding)
+    return op_sharding_sharding.devices_indices_map(global_shape)
+
   @pxla.maybe_cached_property
   def _addressable_device_assignment(self) -> XLADeviceAssignment:
-    process_index = xb.process_index()
-    return [d for d in self._device_assignment if d.process_index == process_index]
+    return [d for d in self._device_assignment
+            if d.process_index == d.client.process_index()]
 
   @functools.lru_cache(maxsize=4096)
   def shard_shape(self, global_shape: Shape) -> Shape:
@@ -186,7 +194,7 @@ class MeshPspecSharding(XLACompatibleSharding):
     _check_mesh_resource_axis(self.mesh, self._parsed_pspec)
 
   def __repr__(self):
-    return f'MeshPspecSharding(mesh={dict(self.mesh.shape)}, partition_spec={self.spec})'
+    return f'NamedSharding(mesh={dict(self.mesh.shape)}, partition_spec={self.spec})'
 
   def __hash__(self):
     if not hasattr(self, '_hash'):
@@ -217,14 +225,6 @@ class MeshPspecSharding(XLACompatibleSharding):
   def device_set(self) -> Set[Device]:
     return set(self.mesh.devices.flat)
 
-  def devices_indices_map(
-      self, global_shape: Shape) -> Mapping[Device, Index]:
-    # TODO(yashkatariya): Remove this when utilities are moved to pxla.py.
-    from jax.experimental import global_device_array
-
-    # `get_shard_indices` is cached.
-    return global_device_array.get_shard_indices(global_shape, self.mesh, self.spec)
-
   @pxla.maybe_cached_property
   def _device_assignment(self) -> XLADeviceAssignment:
     return list(self.mesh.devices.flat)
@@ -253,6 +253,10 @@ class MeshPspecSharding(XLACompatibleSharding):
     return sharding_spec.sharding_proto(special_axes=special_axes)
 
 
+# New name of MeshPspecSharding to match with PositionalSharding below.
+NamedSharding = MeshPspecSharding
+
+
 @functools.lru_cache()
 def _get_replicated_op_sharding():
   proto = xc.OpSharding()
@@ -284,8 +288,7 @@ class SingleDeviceSharding(XLACompatibleSharding):
   def device_set(self) -> Set[Device]:
     return {self._device}
 
-  def devices_indices_map(
-      self, global_shape: Shape) -> Mapping[Device, Index]:
+  def devices_indices_map(self, global_shape: Shape) -> Mapping[Device, Index]:  # type: ignore
     return {self._device: (slice(None),) * len(global_shape)}
 
   @property
@@ -318,15 +321,52 @@ class PmapSharding(XLACompatibleSharding):
       self._hash = hash((tuple(self.devices.flat), self.sharding_spec))
     return self._hash
 
+  def __str__(self):
+    device_ids = [d.id for d in self.devices.flat]
+    return (f'PmapSharding(sharding_spec={self.sharding_spec}, '
+            f'device_ids={device_ids}, '
+            f'device_platform={self.devices.flat[0].platform.upper()}, '
+            f'device_shape={self.devices.shape})')
+
+  def __repr__(self):
+    return (f'PmapSharding(sharding_spec={self.sharding_spec}, '
+            f'devices={self.devices})')
+
+  # TODO(yashkatariya): Expose `sharded_dim_size` in the API if required.
+  @classmethod
+  def default(cls, shape: Shape, sharded_dim: int = 0) -> PmapSharding:
+    """Creates a `PmapSharding` which matches the implicit device order used by
+    `pmap`.
+
+    Args:
+      shape: The shape of the input array.
+      sharded_dim: Dimension the input array is sharded on. Defaults to 0.
+    """
+    # The dtype doesn't matter here. Its only used for creating the
+    # sharding_spec.
+    aval = jax.ShapedArray(shape, np.int32)
+    sharding_spec = pxla._create_pmap_sharding_spec(aval, sharded_dim)
+
+    num_ways_sharded = None
+    for s in sharding_spec.sharding:
+      if isinstance(s, pxla.Unstacked):
+        num_ways_sharded = s.size
+    if num_ways_sharded is None:
+      raise NotImplementedError(
+          '`None` to sharded_dim is not supported. Please file a jax '
+          'issue if you need this feature.')
+
+    pmap_devices = jax.local_devices()[:num_ways_sharded]
+    return cls(pmap_devices, sharding_spec)
+
   @pxla.maybe_cached_property
   def device_set(self) -> Set[Device]:
     return set(self.devices.flat)
 
   @functools.lru_cache(maxsize=4096)
-  def devices_indices_map(
-      self, global_shape: Shape) -> Mapping[Device, Optional[Index]]:
+  def devices_indices_map(self, global_shape: Shape) -> Mapping[Device, Index]:
     indices = pxla.spec_to_indices(global_shape, self.sharding_spec)
-    return {d: i for d, i in safe_zip(self.devices.flat, indices)}  # type: ignore
+    return dict(safe_zip(self.devices.flat, indices))  # type: ignore[arg-type]
 
   @pxla.maybe_cached_property
   def _device_assignment(self) -> XLADeviceAssignment:
@@ -347,12 +387,122 @@ class PmapSharding(XLACompatibleSharding):
     return global_shape[:sharded_dim] + global_shape[sharded_dim+1:]
 
 
-# TODO(yashkatariya): Remove this when minimum_jaxlib version is 0.3.17
-def _hash_op_sharding(op: xc.OpSharding):
-  if op.type == xc.OpSharding.Type.TUPLE:
-    return hash(tuple(_hash_op_sharding(o) for o in  op.tuple_shardings))
-  return hash((tuple(op.tile_assignment_devices), tuple(op.tile_assignment_dimensions),
-               op.type, op.replicate_on_last_tile_dim, tuple(op.last_tile_dims)))
+class PositionalSharding(XLACompatibleSharding):
+  _devices: List[xc.Device]
+  _ids: np.ndarray  # dtype DeviceIdSet
+
+  def __init__(self, devices: Union[Sequence[xc.Device], np.ndarray]):
+    if not isinstance(devices, np.ndarray):
+      devices = np.array(devices, dtype='object')
+    if not devices.size:
+      raise ValueError(f"{self.__class__.__name__}.__init__ requires at least "
+                       f"one device, got {devices}")
+    self._devices = list(devices.flat)
+    name = self._devices[0].platform.upper()
+    self._ids = np.array([DeviceIdSet(name, i) for i in range(devices.size)],
+                         dtype='object').reshape(devices.shape)
+
+  shape = property(op.attrgetter('_ids.shape'))
+  ndim = property(op.attrgetter('_ids.ndim'))
+
+  def __repr__(self) -> str:
+    cls_name = self.__class__.__name__
+    ids = self._ids.copy()
+    platform_name = self._devices[0].platform.upper()
+    for idx, x in np.ndenumerate(ids):
+      ids[idx] = DeviceIdSet(platform_name, *(self._devices[i].id for i in x))
+    body = np.array2string(ids, prefix=cls_name + '(', suffix=')',
+                           max_line_width=100)
+    return f'{cls_name}({body})'
+
+  def reshape(self, *shape):
+    return self.remake(self._devices, self._ids.reshape(*shape))
+
+  def transpose(self, *axes):
+    return self.remake(self._devices, self._ids.transpose(*axes))
+  T = property(transpose)
+
+  def replicate(self, axis=None, keepdims=True):
+    new_ids = self._ids.sum(axis=axis, keepdims=keepdims)  # union
+    return self.remake(self._devices, new_ids)
+
+  @classmethod
+  def remake(
+      cls, devices: List[xc.Device], ids: np.ndarray) -> PositionalSharding:
+    self = cls.__new__(cls)
+    self._devices = devices
+    self._ids = ids
+    return self
+
+  # Hashable
+
+  def __hash__(self) -> int:
+    return id(self._devices)
+
+  def __eq__(self, other) -> bool:
+    return (isinstance(other, PositionalSharding) and
+            id(self._devices) == id(other._devices) and
+            bool(np.all(self._ids == other._ids)))
+
+  # Sharding interface
+
+  @pxla.maybe_cached_property
+  def device_set(self) -> set[xc.Device]:
+    return set(self._devices)
+
+  # XLACompatibleSharding interface
+
+  @functools.lru_cache(maxsize=4096)
+  def _to_xla_op_sharding(self, num_dimensions: int, axis_ctx=None):
+    assert axis_ctx is None
+
+    pbuf = xc.OpSharding()
+    if self.shape == (1,) * self.ndim:
+      pbuf.type = xc.OpSharding.Type.REPLICATED
+      return pbuf
+
+    shape = self.shape[self.ndim - num_dimensions:]  # 'rank promotion' of val
+    set_size, = {len(device_set) for device_set in self._ids.flat}
+    pbuf.type = xc.OpSharding.Type.OTHER
+    if set_size > 1:
+      pbuf.last_tile_dims = [xc.OpSharding.Type.REPLICATED]
+      pbuf.tile_assignment_dimensions = (*shape, set_size)
+    else:
+      pbuf.tile_assignment_dimensions = shape
+    pbuf.tile_assignment_devices = [i for ids in self._ids.flat for i in ids]
+    return pbuf
+
+  @property
+  def _device_assignment(self) -> list[xc.Device]:
+    return self._devices
+
+class DeviceIdSet:
+  _name: str
+  _ids: FrozenSet[int]
+  def __init__(self, name, *ids):
+    self._name = name
+    self._ids = frozenset(ids)
+
+  def __iter__(self):
+    return iter(sorted(self._ids))
+
+  def __add__(self, other) -> DeviceIdSet:
+    assert isinstance(other, DeviceIdSet)
+    return DeviceIdSet(self._name, *(self._ids | other._ids))
+
+  def __len__(self) -> int:
+    return len(self._ids)
+
+  def __repr__(self) -> str:
+    ids = ', '.join(safe_map(str, sorted(self._ids)))
+    return f'{{{self._name} {ids}}}'
+
+  def __hash__(self) -> int:
+    return hash((self._name, self._ids))
+
+  def __eq__(self, other) -> bool:
+    return (isinstance(other, DeviceIdSet) and self._name == other._name and
+            self._ids == other._ids)
 
 
 @pxla.use_cpp_class(xc.OpShardingSharding if xc._version >= 95 else None)
@@ -365,10 +515,7 @@ class OpShardingSharding(XLACompatibleSharding):
 
   @pxla.maybe_cached_property
   def _op_sharding_hash(self):
-    if xla_extension_version >= 81:
-      return hash(xc.HloSharding.from_proto(self._op_sharding))
-    else:
-      return _hash_op_sharding(self._op_sharding)
+    return hash(xc.HloSharding.from_proto(self._op_sharding))
 
   def __eq__(self, other):
     if not isinstance(other, OpShardingSharding):
@@ -384,12 +531,7 @@ class OpShardingSharding(XLACompatibleSharding):
     return self._hash
 
   def __repr__(self):
-    if xla_extension_version >= 96:
-      return f'OpShardingSharding({repr(xc.HloSharding.from_proto(self._op_sharding))})'
-    else:
-      if pxla.is_op_sharding_replicated(self._op_sharding):
-        return 'OpShardingSharding(REPLICATED)'
-      return f'OpShardingSharding({repr(self._op_sharding)})'
+    return f'OpShardingSharding({repr(xc.HloSharding.from_proto(self._op_sharding))})'
 
   def is_compatible_aval(self, aval_shape: Shape):
     num_ways_dim_sharded, _ = pxla._get_num_ways_dim_sharded(self._op_sharding)
@@ -404,8 +546,7 @@ class OpShardingSharding(XLACompatibleSharding):
     return set(self._devices)
 
   @functools.lru_cache(maxsize=4096)
-  def devices_indices_map(
-      self, global_shape: Shape) -> Mapping[Device, Index]:
+  def devices_indices_map(self, global_shape: Shape) -> Mapping[Device, Index]:
     indices = pxla.op_sharding_to_indices(self._op_sharding, global_shape,
                                           len(self._devices))
     return dict(safe_zip(self._devices, indices))
