@@ -16,7 +16,7 @@ from __future__ import annotations
 import dataclasses
 from functools import partial
 
-from typing import Any, Dict, List, Optional, Sequence, Tuple
+from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
 from typing_extensions import Protocol
 
 import numpy as np
@@ -27,8 +27,7 @@ from jax import linear_util as lu
 from jax.interpreters import partial_eval as pe
 from jax._src.util import safe_map, safe_zip, split_list
 
-from jax._src.state.types import (ShapedArrayRef, ReadEffect, WriteEffect,
-                                  AccumEffect)
+from jax._src.state.types import ShapedArrayRef
 from jax._src.state.primitives import get_p, swap_p, addupdate_p
 
 ## JAX utilities
@@ -43,13 +42,17 @@ zip, unsafe_zip = safe_zip, zip
 # into a "pure" jaxpr that takes in and outputs values and no longer has the
 # `Read/Write/Accum` effects.
 
-def discharge_state(jaxpr: core.Jaxpr, consts: Sequence[Any]
+def discharge_state(jaxpr: core.Jaxpr, consts: Sequence[Any], * ,
+                    should_discharge: Union[bool, Sequence[bool]] = True
                     ) -> Tuple[core.Jaxpr, List[Any]]:
   """Converts a jaxpr that takes in `Ref`s into one that doesn't."""
+  if isinstance(should_discharge, bool):
+    should_discharge = [should_discharge] * len(jaxpr.invars)
   in_avals = [core.ShapedArray(v.aval.shape, v.aval.dtype)
-              if type(v.aval) is ShapedArrayRef
-              else v.aval for v in jaxpr.invars]
-  eval_jaxpr = lu.wrap_init(partial(_eval_jaxpr_discharge_state, jaxpr, consts))
+              if type(v.aval) is ShapedArrayRef and d
+              else v.aval for v, d in zip(jaxpr.invars, should_discharge)]
+  eval_jaxpr = lu.wrap_init(partial(_eval_jaxpr_discharge_state, jaxpr,
+                                    should_discharge, consts))
   new_jaxpr, _ , new_consts = pe.trace_to_jaxpr_dynamic(eval_jaxpr, in_avals)
   return new_jaxpr, new_consts
 
@@ -68,7 +71,8 @@ class Environment:
 
 class DischargeRule(Protocol):
 
-  def __call__(self, in_avals: Sequence[core.AbstractValue], *args: Any,
+  def __call__(self, in_avals: Sequence[core.AbstractValue],
+      out_avals: Sequence[core.AbstractValue], *args: Any,
       **params: Any) -> Tuple[Sequence[Optional[Any]], Sequence[Any]]:
     ...
 
@@ -82,8 +86,9 @@ def register_discharge_rule(prim: core.Primitive):
 def _has_refs(eqn: core.JaxprEqn):
   return any(isinstance(v.aval, ShapedArrayRef) for v in eqn.invars)
 
-def _eval_jaxpr_discharge_state(jaxpr: core.Jaxpr, consts: Sequence[Any],
-                                *args: Any):
+def _eval_jaxpr_discharge_state(
+    jaxpr: core.Jaxpr, should_discharge: Sequence[bool], consts: Sequence[Any],
+    *args: Any):
   env = Environment({})
 
   map(env.write, jaxpr.constvars, consts)
@@ -91,18 +96,24 @@ def _eval_jaxpr_discharge_state(jaxpr: core.Jaxpr, consts: Sequence[Any],
   # regular values in this interpreter.
   map(env.write, jaxpr.invars, args)
 
+  refs_to_discharge = set(id(v.aval) for v, d
+                          in zip(jaxpr.invars, should_discharge) if d
+                          and isinstance(v.aval, ShapedArrayRef))
+
   for eqn in jaxpr.eqns:
-    if _has_refs(eqn):
+    if _has_refs(eqn) and any(id(v.aval) in refs_to_discharge
+                              for v in eqn.invars):
       if eqn.primitive not in _discharge_rules:
         raise NotImplementedError("No state discharge rule implemented for "
             f"primitive: {eqn.primitive}")
       invals = map(env.read, eqn.invars)
       in_avals = [v.aval for v in eqn.invars]
-      new_invals, ans = _discharge_rules[eqn.primitive](in_avals, *invals,
-                                                        **eqn.params)
+      out_avals = [v.aval for v in eqn.outvars]
+      new_invals, ans = _discharge_rules[eqn.primitive](
+          in_avals, out_avals, *invals, **eqn.params)
       for new_inval, invar in zip(new_invals, eqn.invars):
         if new_inval is not None:
-          env.write(invar, new_inval)
+          env.write(invar, new_inval)  # type: ignore[arg-type]
     else:
       # Default primitive rule, similar to `core.eval_jaxpr`. Note that here
       # we assume any higher-order primitives inside of the jaxpr are *not*
@@ -119,12 +130,15 @@ def _eval_jaxpr_discharge_state(jaxpr: core.Jaxpr, consts: Sequence[Any],
   # them up by looking at `len(jaxpr.outvars)`.
   out_vals = map(env.read, jaxpr.outvars)
   ref_vals = map(
-      env.read, [v for v in jaxpr.invars if type(v.aval) is ShapedArrayRef])
+      env.read, [v for v in jaxpr.invars if id(v.aval) in refs_to_discharge])
   return out_vals + ref_vals
 
 @register_discharge_rule(get_p)
-def _get_discharge_rule(_: Sequence[core.AbstractValue], x, *non_slice_idx,
-                        indexed_dims: Sequence[bool]):
+def _get_discharge_rule(
+    in_avals: Sequence[core.AbstractValue],
+    out_avals: Sequence[core.AbstractValue], x, *non_slice_idx,
+    indexed_dims: Sequence[bool]):
+  del in_avals, out_avals
   y = _get_discharge(x, non_slice_idx, indexed_dims)
   return (None,) * (len(non_slice_idx) + 1), y
 
@@ -154,8 +168,11 @@ def _indexer(idx, indexed_dims):
   return indexer
 
 @register_discharge_rule(swap_p)
-def _swap_discharge_rule(_: Sequence[core.AbstractValue], x, val, *non_slice_idx,
-                         indexed_dims: Sequence[bool]):
+def _swap_discharge_rule(
+    in_avals: Sequence[core.AbstractValue],
+    out_avals: Sequence[core.AbstractValue], x, val, *non_slice_idx,
+    indexed_dims: Sequence[bool]):
+  del in_avals, out_avals
   if not any(indexed_dims):
     z, x_new = x, val
   z, x_new = _swap_discharge(x, val, non_slice_idx, indexed_dims)
@@ -173,8 +190,11 @@ def _swap_discharge(x, val, idx, indexed_dims):
   return z, x_new
 
 @register_discharge_rule(addupdate_p)
-def _addupdate_discharge_rule(_: Sequence[core.AbstractValue], x, val,
-                              *non_slice_idx, indexed_dims: Sequence[bool]):
+def _addupdate_discharge_rule(
+    in_avals: Sequence[core.AbstractValue],
+    out_avals: Sequence[core.AbstractValue], x, val, *non_slice_idx,
+    indexed_dims: Sequence[bool]):
+  del in_avals, out_avals
   ans = _addupdate_discharge(x, val, non_slice_idx, indexed_dims)
   return (ans, None) + (None,) * len(non_slice_idx), []
 
@@ -205,8 +225,9 @@ def _dynamic_update_index(x, idx, val, indexed_dims):
   return lax.dynamic_update_slice(x, val.reshape(sizes), starts)
 
 @register_discharge_rule(core.closed_call_p)
-def _closed_call_discharge_rule(in_avals: Sequence[core.AbstractValue], *args,
-                                call_jaxpr: core.ClosedJaxpr):
+def _closed_call_discharge_rule(
+    in_avals: Sequence[core.AbstractValue], _,*args,
+    call_jaxpr: core.ClosedJaxpr):
   jaxpr, consts = call_jaxpr.jaxpr, call_jaxpr.consts
   num_outs = len(jaxpr.outvars)
   discharged_jaxpr, discharged_consts = discharge_state(jaxpr, consts)
