@@ -20,7 +20,7 @@ from functools import partial
 import re
 import os
 import textwrap
-from typing import Callable, Dict, List, Generator, Sequence, Tuple, Union
+from typing import Callable, List, Generator, Sequence, Tuple, Union
 import unittest
 import warnings
 import zlib
@@ -255,6 +255,9 @@ def is_device_rocm():
 def is_device_cuda():
   return xla_bridge.get_backend().platform_version.startswith('cuda')
 
+def is_cloud_tpu():
+  return 'libtpu' in xla_bridge.get_backend().platform_version
+
 def is_device_tpu_v4():
   return jax.devices()[0].device_kind == "TPU v4"
 
@@ -658,20 +661,6 @@ def assert_dot_precision(expected_precision, fun, *args):
       assert precision == expected_precision, msg
 
 
-_CACHED_INDICES: Dict[int, Sequence[int]] = {}
-
-def cases_from_list(xs):
-  xs = list(xs)
-  n = len(xs)
-  k = min(n, FLAGS.num_generated_cases)
-  # Random sampling for every parameterized test is expensive. Do it once and
-  # cache the result.
-  indices = _CACHED_INDICES.get(n)
-  if indices is None:
-    rng = npr.RandomState(42)
-    _CACHED_INDICES[n] = indices = rng.permutation(n)
-  return [xs[i] for i in indices[:k]]
-
 def cases_from_gens(*gens):
   sizes = [1, 3, 10]
   cases_per_size = int(FLAGS.num_generated_cases / len(sizes)) + 1
@@ -703,14 +692,20 @@ def named_cases_from_sampler(gen):
     yield case
 
 
+# Random sampling for every parameterized test is expensive. Do it once and
+# cache the result.
+@functools.lru_cache(maxsize=None)
+def _choice(n, m):
+  rng = np.random.RandomState(42)
+  return rng.choice(n, size=m, replace=False)
+
 def sample_product_testcases(*args, **kw):
   """Non-decorator form of sample_product."""
   args = [list(arg) for arg in args]
   kw = [(k, list(v)) for k, v in kw.items()]
   n = prod(len(a) for a in args) * prod(len(v) for _, v in kw)
-  rng = np.random.RandomState(42)
   testcases = []
-  for i in rng.choice(n, size=min(n, FLAGS.num_generated_cases), replace=False):
+  for i in _choice(n, min(n, FLAGS.num_generated_cases)):
     testcase = {}
     for a in args:
       testcase.update(a[i % len(a)])
@@ -880,6 +875,12 @@ class JaxTestCase(parameterized.TestCase):
     self.assertMultiLineEqual(expected_clean, what_clean,
                               msg=f"Found\n{what}\nExpecting\n{expected}")
 
+  @contextmanager
+  def assertNoWarnings(self):
+    with warnings.catch_warnings(record=True) as caught_warnings:
+      yield
+    self.assertEmpty(caught_warnings)
+
   def _CompileAndCheck(self, fun, args_maker, *, check_dtypes=True,
                        rtol=None, atol=None, check_cache_misses=True):
     """Helper method for running JAX compilation and allclose assertions."""
