@@ -12,35 +12,64 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-
+from collections import namedtuple
 import functools
 from functools import partial
 import itertools as it
-from collections import namedtuple
+import logging
 import operator
 import types
-import threading
-from typing import (Any, Callable, Dict, Iterable, List, Tuple, Generic,
-                    TypeVar, Set, Iterator, Sequence, Optional)
-import weakref
+from typing import (Any, Callable, Generic, Iterable, Iterator, List,
+                    Optional, Sequence, Set, Tuple, TypeVar, overload)
 
-from absl import logging
 import numpy as np
 
 from jax._src.lib import xla_client as xc
-from jax._src.lib import xla_extension_version
 from jax.config import config
 
+logger = logging.getLogger(__name__)
+
 Seq = Sequence
 
 T = TypeVar("T")
+T1 = TypeVar("T1")
+T2 = TypeVar("T2")
+T3 = TypeVar("T3")
+
+# safe_zip cannot yet be fully annotated, so we use a strategy similar
+# to that used for builtins.zip in python/typeshed. This supports
+# return types matching input types for up to three arguments.
+@overload
+def safe_zip(__arg1: Iterable[T1]) -> List[Tuple[T1]]: ...
+@overload
+def safe_zip(__arg1: Iterable[T1], __arg2: Iterable[T2]) -> List[Tuple[T1, T2]]: ...
+@overload
+def safe_zip(__arg1: Iterable[T1], __arg2: Iterable[T2], __arg3: Iterable[T3]) -> List[Tuple[T1, T2, T3]]: ...
+@overload
+def safe_zip(__arg1: Iterable[Any], __arg2: Iterable[Any], __arg3: Iterable[Any], __arg4: Iterable[Any], *args) -> List[Tuple[Any, ...]]: ...
 
 def safe_zip(*args):
+  args = list(map(list, args))
   n = len(args[0])
   for arg in args[1:]:
     assert len(arg) == n, f'length mismatch: {list(map(len, args))}'
   return list(zip(*args))
 
+# safe_map cannot yet be fully annotated, so we use a strategy similar
+# to that used for builtins.map in python/typeshed. This supports
+# checking input types for the callable with up to three arguments.
+@overload
+def safe_map(f: Callable[[T1], T], __arg1: Iterable[T1]) -> List[T]: ...
+
+@overload
+def safe_map(f: Callable[[T1, T2], T], __arg1: Iterable[T1], __arg2: Iterable[T2]) -> List[T]: ...
+
+@overload
+def safe_map(f: Callable[[T1, T2, T3], T], __arg1: Iterable[T1], __arg2: Iterable[T2], __arg3: Iterable[T3]) -> List[T]: ...
+
+@overload
+def safe_map(f: Callable[..., T], __arg1: Iterable[Any], __arg2: Iterable[Any], __arg3: Iterable[Any], __arg4: Iterable[Any], *args) -> List[T]: ...
+
 def safe_map(f, *args):
   args = list(map(list, args))
   n = len(args[0])
@@ -48,24 +77,26 @@ def safe_map(f, *args):
     assert len(arg) == n, f'length mismatch: {list(map(len, args))}'
   return list(map(f, *args))
 
-def unzip2(xys):
+def unzip2(xys: Iterable[Tuple[T1, T2]]
+    ) -> Tuple[Tuple[T1, ...], Tuple[T2, ...]]:
   """Unzip sequence of length-2 tuples into two tuples."""
   # Note: we deliberately don't use zip(*xys) because it is lazily evaluated,
   # is too permissive about inputs, and does not guarantee a length-2 output.
-  xs = []
-  ys = []
+  xs: List[T1] = []
+  ys: List[T2] = []
   for x, y in xys:
     xs.append(x)
     ys.append(y)
   return tuple(xs), tuple(ys)
 
-def unzip3(xyzs):
+def unzip3(xyzs: Iterable[Tuple[T1, T2, T3]]
+    ) -> Tuple[Tuple[T1, ...], Tuple[T2, ...], Tuple[T3, ...]]:
   """Unzip sequence of length-3 tuples into three tuples."""
   # Note: we deliberately don't use zip(*xyzs) because it is lazily evaluated,
   # is too permissive about inputs, and does not guarantee a length-3 output.
-  xs = []
-  ys = []
-  zs = []
+  xs: List[T1] = []
+  ys: List[T2] = []
+  zs: List[T3] = []
   for x, y, z in xyzs:
     xs.append(x)
     ys.append(y)
@@ -238,76 +269,7 @@ def weakref_lru_cache(call: Callable, maxsize=2048):
   and strong refs to all subsequent operations. In all other respects it should
   behave similar to `functools.lru_cache`.
   """
-  if xla_extension_version >= 87:
-    return xc.weakref_lru_cache(config._trace_context, call, maxsize)
-  cache: Dict[Any, Any] = {}
-  hits = misses = 0
-  lock = threading.Lock()
-
-  def remove_key(tctx, args, kwargs, weak_arg):
-    k = (weak_arg, tctx, args, kwargs)
-    try:
-      # This has a chance to race with the iteration in next(iter(cache)),
-      # but we cannot lock because GC can get triggered synchronously inside
-      # a critical section and will not relinquish control until the callback
-      # has finished. This would lead to a deadlock between this weakref
-      # cleanup function and any function below which locks.
-      del cache[k]
-    except KeyError:
-      pass
-
-  def wrapped(weak_arg, *args, **kwargs):
-    nonlocal hits, misses
-    if config.jax_check_tracer_leaks:
-      return call(weak_arg, *args, **kwargs)
-    kwargs_key = tuple(kwargs.items())
-    tctx = config._trace_context()
-    k = (weakref.ref(weak_arg,
-         functools.partial(remove_key, tctx, args, kwargs_key)),
-         tctx, args, kwargs_key)
-    with lock:
-      if k in cache:
-        hits += 1
-        result = cache[k]
-        # del and reinsert to bump key in the insertion order.
-        del cache[k]
-        cache[k] = result
-        return result
-      misses += 1
-    result = call(weak_arg, *args, **kwargs)
-    with lock:
-      cache[k] = result
-      num_errors = 0
-      while len(cache) > maxsize:
-        try:
-          del_k = next(iter(cache))
-          # This happens if a weakref callback happens between iter and
-          # next. Just ignore the error. WeakKeyDictionary handles this
-          # by deferring the deletes, but that has a chance at leaking,
-          # and this solution is easier.
-        except RuntimeError:
-          num_errors += 1
-          if num_errors > len(cache):
-            # This must be some other problem.
-            raise
-          else:
-            continue
-        del cache[del_k]
-      return result
-
-  def cache_info():
-    with lock:
-      return CacheInfo(hits, misses, maxsize, len(cache))
-
-  def cache_clear():
-    nonlocal hits, misses
-    with lock:
-      hits = misses = 0
-      cache.clear()
-
-  wrapped.cache_info = cache_info
-  wrapped.cache_clear = cache_clear
-  return wrapped
+  return xc.weakref_lru_cache(config._trace_context, call, maxsize)
 
 def prod(xs):
   out = 1
@@ -510,7 +472,7 @@ def distributed_debug_log(*pairs):
       lines.append("DISTRIBUTED_DEBUG logging failed!")
       lines.append(f"{e}")
     lines.append("DISTRIBUTED_DEBUG_END")
-    logging.warning("\n".join(lines))
+    logger.warning("\n".join(lines))
 
 
 class OrderedSet(Generic[T]):
