@@ -13,17 +13,15 @@
 # limitations under the License.
 
 import os
-import pathlib
 
 from jax.experimental.compilation_cache.cache_interface import CacheInterface
-from etils import epath
-from absl import logging
+from jax._src import path as pathlib
 
 class GFileCache(CacheInterface):
 
   def __init__(self, path: str):
     """Sets up a cache at 'path'. Cached values may already be present."""
-    self._path = epath.Path(path)
+    self._path = pathlib.Path(path)
     self._path.mkdir(parents=True, exist_ok=True)
 
   def get(self, key: str):
