@@ -21,14 +21,12 @@ import unittest
 
 from absl import logging
 from absl.testing import absltest
-from absl.testing import parameterized
 
 import jax
 from jax import ad_checkpoint
 from jax import dtypes
 from jax import lax
 from jax import numpy as jnp
-from jax._src import lib as jaxlib
 from jax._src import source_info_util
 from jax._src import test_util as jtu
 import jax._src.lib.xla_bridge
@@ -51,12 +49,6 @@ config.parse_flags_with_absl()
 
 class Jax2TfTest(tf_test_util.JaxToTfTestCase):
 
-  def setUp(self):
-    super().setUp()
-    # TODO(b/252943725): re-enable these tests
-    if config.jax_array and config.jax2tf_default_experimental_native_lowering:
-      raise unittest.SkipTest("Test disabled for JAX_ARRAY")
-
   def test_empty(self):
     f_jax = lambda x, y: x
     self.ConvertAndCompare(f_jax, 0.7, 1)
@@ -200,12 +192,10 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
 
     self.assertIsNotNone(f_tf.get_concrete_function())
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-    dict(testcase_name=f"_dtype={dtype.__name__}_function={with_function}",
-         dtype=dtype,
-         with_function=with_function)
-    for dtype in [np.int64, np.float64]
-    for with_function in [True, False]))
+  @jtu.sample_product(
+    dtype=[np.int64, np.float64],
+    with_function=[True, False],
+  )
   def test_converts_64bit(self, dtype=np.int64, with_function=False):
     if not config.jax_enable_x64:
       self.skipTest("requires x64 mode")
@@ -221,40 +211,46 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(jnp.sin(big_const),
                         f_conv(tf.constant(big_const)))
 
-  def test_64bit_behavior_enable_x64(self):
+  def test_64bit_behavior_enable_x64_readme(self):
+    # Tests some of the examples from the README
     if not config.jax_enable_x64:
       self.skipTest("requires x64 mode")
 
     # JAX and TF have different default float types if JAX_ENABLE_X64=1
-    self.assertEqual(tf.math.sin(0.7).dtype, tf.float32)
-    self.assertEqual(jnp.sin(0.7).dtype, jnp.float64)
+    self.assertEqual(tf.math.sin(3.14).dtype, tf.float32)
+    self.assertEqual(jnp.sin(3.14).dtype, jnp.float64)
 
     # jax2tf.convert has the same behavior as JAX
-    self.assertEqual(jax2tf.convert(jnp.sin)(0.7).dtype, tf.float64)
+    self.assertEqual(jax2tf.convert(jnp.sin)(3.14).dtype, tf.float64)
+    # The following will compute `sin` in float64.
+    self.assertEqual(tf.function(jax2tf.convert(jnp.sin))(tf.Variable(3.14, dtype=tf.float64)).dtype, tf.float64)
 
-  def test_64bit_behavior_not_enable_x64(self):
+    # The following will compute `sin` in float32.
+    self.assertEqual(tf.function(jax2tf.convert(jnp.sin))(tf.Variable(3.14)).dtype, tf.float32)
+
+  def test_64bit_behavior_not_enable_x64_readme(self):
+    # Tests some of the examples from the README
     if config.jax_enable_x64:
       self.skipTest("requires not x64 mode")
 
-    # JAX and TF have same default float types if JAX_ENABLE_X64=1
-    self.assertEqual(tf.math.sin(0.7).dtype, tf.float32)
-    self.assertEqual(jnp.sin(0.7).dtype, jnp.float32)
+    # JAX and TF have same default float types if JAX_ENABLE_X64=0
+    self.assertEqual(tf.math.sin(3.14).dtype, tf.float32)
+    self.assertEqual(jnp.sin(3.14).dtype, jnp.float32)
 
-    # Except that JAX forces values to 32-bit
-    self.assertEqual(jnp.sin(np.float64(0.7)).dtype, jnp.float32)
+    self.assertEqual(tf.math.sin(np.float64(3.14)).dtype, tf.float64)
+    # JAX forces values to 32-bit
+    self.assertEqual(jnp.sin(np.float64(3.14)).dtype, jnp.float32)
 
     # jax2tf.convert has the same behavior as JAX
-    self.assertEqual(jax2tf.convert(jnp.sin)(0.7).dtype, tf.float32)
-    self.assertEqual(jax2tf.convert(jnp.sin)(np.float64(0.7)).dtype, tf.float32)
+    self.assertEqual(jax2tf.convert(jnp.sin)(3.14).dtype, tf.float32)
+    self.assertEqual(jax2tf.convert(jnp.sin)(np.float64(3.14)).dtype, tf.float32)
+    self.assertEqual(tf.function(jax2tf.convert(jnp.sin))(tf.Variable(3.14, dtype=tf.float64)).dtype, tf.float32)
 
   def test_function(self):
     f_jax = jax.jit(lambda x: jnp.sin(jnp.cos(x)))
     self.ConvertAndCompare(f_jax, 0.7)
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-      dict(testcase_name=f"function={with_function}",
-           with_function=with_function)
-      for with_function in [False, True]))
+  @jtu.sample_product(with_function=[False, True])
   def test_gradients_disabled(self, with_function=False):
     f_tf = jax2tf.convert(jnp.tan, with_gradient=False)
     if with_function:
@@ -270,10 +266,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
         y = f_tf(x)
         _ = tape.gradient(y, x)
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-    dict(testcase_name=f"function={with_function}",
-         with_function=with_function)
-    for with_function in [False, True]))
+  @jtu.sample_product(with_function=[False, True])
   def test_gradients(self, with_function=True):
     def f(x, y):
       return x * x, x * y
@@ -291,10 +284,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(5., tape.gradient(v, x))
     self.assertAllClose(4., tape.gradient(v, y))
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-    dict(testcase_name=f"function={with_function}",
-         with_function=with_function)
-    for with_function in [False, True]))
+  @jtu.sample_product(with_function=[False, True])
   def test_gradients_pytree(self, with_function=True):
     def f(xy: Tuple[float, float]) -> Dict[str, float]:
       x, y = xy
@@ -368,10 +358,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(grad_jax.b, grad_tf[1])
 
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-    dict(testcase_name=f"function={with_function}",
-         with_function=with_function)
-    for with_function in [False, True]))
+  @jtu.sample_product(with_function=[False, True])
   def test_gradients_with_ordered_dict_input(self, with_function=True):
     def f(inputs):
       out = 0.0
@@ -394,10 +381,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(np.array([1.]), tape.gradient(u, x).numpy())
     self.assertAllClose(np.array([1., 1.]), tape.gradient(u, y).numpy())
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-      dict(testcase_name=f"function={with_function}",
-           with_function=with_function)
-      for with_function in [False, True]))
+  @jtu.sample_product(with_function=[False, True])
   def test_gradients_with_custom_jvp(self, with_function=True):
     """Check gradients, for a function with custom JVP."""
     @jax.custom_jvp
@@ -428,10 +412,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(4. * 4., y)
     self.assertAllClose(3. * 4., tape.gradient(y, x))
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-      dict(testcase_name=f"function={with_function}",
-           with_function=with_function)
-      for with_function in [False, True]))
+  @jtu.sample_product(with_function=[False, True])
   def test_gradients_with_custom_vjp(self, with_function=True):
     """Check gradients, for a function with custom VJP."""
     @jax.custom_vjp
@@ -498,10 +479,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(jnp.zeros(np.shape(d_dx_jax), np.int32),
                         d_dx_tf.numpy())
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-      dict(testcase_name=f"function={with_function}",
-           with_function=with_function)
-      for with_function in [False, True]))
+  @jtu.sample_product(with_function=[False, True])
   def test_gradients_unused_argument_readme(self, with_function=False):
     # x1 and x3 are not used. x3 has integer type.
     def fn(x0, x1, x2, x3):
@@ -546,10 +524,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(g_jax2tf[2].numpy(), np.float32(2.))
     self.assertAllClose(g_jax2tf[3].numpy(), np.int32(0))
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-      dict(testcase_name=f"function={with_function}",
-           with_function=with_function)
-      for with_function in [False, True]))
+  @jtu.sample_product(with_function=[False, True])
   def test_gradients_int_argument(self, with_function=False):
     # https://github.com/google/jax/issues/6975
     # Also issue #6975.
@@ -792,21 +767,15 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     # The computation of grad_f computes "sin" 5 times, 3 for the forward pass
     # and then to rematerialize "x2" and "x3" in the backward pass.
     arg = np.array(3.)
-    # Check that we have a Sin under a conditional
-    f_tf = tf.function(jax2tf.convert(jax.grad(remat_f)), autograph=False)
-    f_tf_graph = f_tf.get_concrete_function(arg).graph.as_graph_def()
+    f_tf = jax2tf.convert(jax.grad(remat_f))
+    f_tf_hlo = self.TfToHlo(f_tf, arg)
     if jax.config.jax_remat_opt_barrier:
-      if config.jax2tf_default_experimental_native_lowering:
-        self.assertRegex(
-          str(f_tf_graph), r"mhlo.optimization_barrier")
-      else:
-        self.assertRegex(
-            str(f_tf_graph), r"XlaOptimizationBarrier")
+      self.assertRegex(f_tf_hlo, r"opt-barrier")
     elif config.jax_experimental_name_stack:
-      self.assertRegex(str(f_tf_graph),
+      self.assertRegex(f_tf_hlo,
                        r'transpose/jax2tf_f_/jvp/checkpoint/cond/branch_1_fun/Sin')
     else:
-      self.assertRegex(str(f_tf_graph),
+      self.assertRegex(f_tf_hlo,
                        r'switch_case/indexed_case/Sin')
 
   def test_remat_free_var(self):
@@ -832,12 +801,11 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     # in JAX prior to conversion).
     def f_jax():
       return jnp.sin(1.)
-    f_tf = tf.function(jax2tf.convert(f_jax), autograph=False)
-    f_tf_graph = f_tf.get_concrete_function().graph.as_graph_def()
-    if config.jax2tf_default_experimental_native_lowering:
-      self.assertIn("mhlo.sine", str(f_tf_graph))
-    else:
-      self.assertIn('op: "Sin"', str(f_tf_graph))
+    f_tf = jax2tf.convert(f_jax)
+    # for native lowering the HLO we get from TF is constant-folded, so this
+    # test fails.
+    if not config.jax2tf_default_experimental_native_lowering:
+      self.assertIn("sine(", self.TfToHlo(f_tf))
 
   def test_convert_of_nested_independent_jit(self):
     def func(x):
@@ -874,10 +842,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
         ValueError, "convert must be used outside all JAX transformations"):
       jax2tf.convert(outer)(2.)
 
-
-  @parameterized.named_parameters(jtu.cases_from_list(
-    dict(testcase_name=f"_{transform}", transform=transform)
-    for transform in ["jit", "jvp", "grad", "vmap"]))
+  @jtu.sample_product(transform=["jit", "jvp", "grad", "vmap"])
   def test_convert_under_transform_error(self, transform="vmap"):
     def outer(y):
       return jax2tf.convert(jnp.sin)(y)  # Inner convert takes tracer args
@@ -886,9 +851,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
         ValueError, "convert must be used outside all JAX transformations"):
       self.TransformConvertAndCompare(outer, np.ones((4,)), transform)
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-    dict(testcase_name=f"_{transform}", transform=transform)
-    for transform in ["jit", "jvp", "grad", "vmap"]))
+  @jtu.sample_product(transform=["jit", "jvp", "grad", "vmap"])
   def test_convert_under_transform_error_non_tracer(self, transform="vmap"):
     def outer(y):
       sin_1 = jax2tf.convert(jnp.sin)(1.)  # Inner convert takes non-tracer arg
@@ -899,23 +862,21 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
       self.TransformConvertAndCompare(outer, np.ones((4,)), transform)
 
   def test_name_scope(self):
-    @tf.function(autograph=False)
-    def run():
+    def run_tf():
       @jax.named_call
-      def my_test_function(x):
+      def my_test_function_jax(x):
         return x * x
 
-      def caller(x):
-        return my_test_function(jnp.sin(x))
+      def caller_jax(x):
+        return my_test_function_jax(jnp.sin(x))
 
-      out = jax2tf.convert(caller, with_gradient=False)(2.)
+      out = jax2tf.convert(caller_jax, with_gradient=False)(2.)
       return out
-    run_graph = run.get_concrete_function().graph.as_graph_def()
-    print(str(run_graph))
     if config.jax2tf_default_experimental_native_lowering:
-      self.assertIn("my_test_function/mul", str(run_graph))
+      self.assertIn("my_test_function_jax/mul", self.TfToHlo(run_tf))
     else:
-      self.assertIn("my_test_function/jit_fn_/Mul", str(run_graph))
+      self.assertIn("my_test_function_jax/jit_fn_/Mul",
+                    str(tf.function(run_tf, autograph=False).get_concrete_function().graph.as_graph_def()))
 
   def test_bfloat16_constant(self):
     # Re: https://github.com/google/jax/issues/3942
@@ -940,6 +901,8 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
   def test_shared_constants(self):
     # Check that the constants are shared properly in converted functions
     # See https://github.com/google/jax/issues/7992.
+    if config.jax2tf_default_experimental_native_lowering:
+      raise unittest.SkipTest("shared constants tests not interesting for native lowering")
     const = np.random.uniform(size=256).astype(np.float32)  # A shared constant
     def f(x):
       return x + const + const + const + const
@@ -950,6 +913,8 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
   def test_shared_constants_under_cond(self):
     # Check that the constants are shared properly in converted functions
     # See https://github.com/google/jax/issues/7992.
+    if config.jax2tf_default_experimental_native_lowering:
+      raise unittest.SkipTest("shared constants tests not interesting for native lowering")
     const = np.random.uniform(size=256).astype(np.float32)  # A shared constant
     x = np.ones((256,), dtype=np.float32)
     def f1(x):
@@ -962,6 +927,8 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
 
   def test_shared_constants_under_scan(self):
     # See https://github.com/google/jax/issues/7992.
+    if config.jax2tf_default_experimental_native_lowering:
+      raise unittest.SkipTest("shared constants tests not interesting for native lowering")
     const = np.random.uniform(size=256).astype(np.float32)  # A shared constant
     xs = np.ones((8, 256), dtype=np.float32)
     def f1(xs):
@@ -978,6 +945,8 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
 
   def test_shared_constants_under_jit(self):
     # We do not share constants under jit.
+    if config.jax2tf_default_experimental_native_lowering:
+      raise unittest.SkipTest("shared constants tests not interesting for native lowering")
     const = np.random.uniform(size=(16, 16)).astype(np.float32)  # A shared constant
     @jax.jit
     def g_jit(x):
@@ -997,10 +966,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(tf_fn(tf.constant(1.375, tf.bfloat16)).numpy(),
                         jnp.bfloat16(2.750))
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-      dict(testcase_name=f"function={with_function}",
-           with_function=with_function)
-      for with_function in [False, True]))
+  @jtu.sample_product(with_function=[False, True])
   def test_kwargs(self, with_function=True):
     # Re: https://github.com/google/jax/issues/6791
     def f_jax(*, x):
@@ -1012,10 +978,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
       f_tf(x=np.zeros(3, dtype=np.float32)),  # Call with kwargs.
       np.zeros((), dtype=np.float32))
 
-  @parameterized.named_parameters(jtu.cases_from_list(
-      dict(testcase_name=f"function={with_function}",
-           with_function=with_function)
-      for with_function in [False, True]))
+  @jtu.sample_product(with_function=[False, True])
   def test_grad_kwargs(self, with_function=False):
     # Re: https://github.com/google/jax/issues/6791
     x = (np.zeros(3, dtype=np.float32),
@@ -1085,7 +1048,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
         f_simple, x,
         [tf_test_util.OpMetadataGraph(tf_type="Sin",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 2,
+                                      source_line=user_frame.start_line + 2,
                                       op_name="jax2tf(f_simple)/sin",
                                       op_type="sin")
          ]
@@ -1109,17 +1072,17 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
         f_caller, x,
         [tf_test_util.OpMetadataGraph(tf_type="Tanh",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 4,
+                                      source_line=user_frame.start_line + 4,
                                       op_name="jax2tf(f_caller)/tanh",
                                       op_type="tanh"),
          tf_test_util.OpMetadataGraph(tf_type="Cos",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 2,
+                                      source_line=user_frame.start_line + 2,
                                       op_name="jax2tf(f_caller)/jit(f_callee)/cos",
                                       op_type="cos"),
          tf_test_util.OpMetadataGraph(tf_type="Sin",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 6,
+                                      source_line=user_frame.start_line + 6,
                                       op_name="jax2tf(f_caller)/sin",
                                       op_type="sin"),
          ]
@@ -1143,17 +1106,17 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
         f_caller, x,
         [tf_test_util.OpMetadataGraph(tf_type="Tanh",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 4,
+                                      source_line=user_frame.start_line + 4,
                                       op_name="jax2tf(f_caller)/tanh",
                                       op_type="tanh"),
          tf_test_util.OpMetadataGraph(tf_type="Cos",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 2,
+                                      source_line=user_frame.start_line + 2,
                                       op_name="jax2tf(f_caller)/named(callee)/cos",
                                       op_type="cos"),
          tf_test_util.OpMetadataGraph(tf_type="Sin",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 6,
+                                      source_line=user_frame.start_line + 6,
                                       op_name="jax2tf(f_caller)/sin",
                                       op_type="sin"),
          ]
@@ -1184,17 +1147,17 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
         f_while_cond, x,
         [tf_test_util.OpMetadataGraph(tf_type="Cos",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 5,
+                                      source_line=user_frame.start_line + 5,
                                       op_name="jax2tf(f_while_cond)/while/body/cos",
                                       op_type="cos"),
          tf_test_util.OpMetadataGraph(tf_type="Sin",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 7,
+                                      source_line=user_frame.start_line + 7,
                                       op_name="jax2tf(f_while_cond)/while/body/branch_1_fun/sin",
                                       op_type="sin"),
          tf_test_util.OpMetadataGraph(tf_type="FloorMod",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 6,
+                                      source_line=user_frame.start_line + 6,
                                       op_name="jax2tf(f_while_cond)/while/body/rem",
                                       op_type="rem"),
          ]
@@ -1229,12 +1192,12 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
         f_while, x,
         [tf_test_util.OpMetadataGraph(tf_type="Sin",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 4,
+                                      source_line=user_frame.start_line + 4,
                                       op_name="jax2tf(f_while)/while/body/sin",
                                       op_type="sin"),
          tf_test_util.OpMetadataGraph(tf_type="LessEqual",
                                       source_file=__file__,
-                                      source_line=user_frame.line_num + 8,
+                                      source_line=user_frame.start_line + 8,
                                       op_name="jax2tf(f_while)/while/body_pred/le",
                                       op_type="le"),
          ]
@@ -1268,8 +1231,6 @@ def get_serialized_computation(
     lowered = jax.jit(f_jax, abstracted_axes=abstracted_axes).lower(*args)
   mhlo_module = lowered.compiler_ir(dialect='mhlo')
   mhlo_module_text = mlir.module_to_string(mhlo_module)
-  if jaxlib.version <= (0, 3, 14):
-    mhlo_module_text = jax2tf.jax2tf._fixup_mhlo_module_text(mhlo_module_text)
   logging.info(f'Serialized ir.Module = {mhlo_module_text}')
   return mhlo_module_text
 
