@@ -30,6 +30,7 @@ from jax import core
 from jax import linear_util as lu
 from jax import stages
 from jax._src import array
+from jax._src.stages import CompiledCallParams
 from jax._src.api import _check_callable, _check_arg, FLAGS, device_put
 from jax._src.config import config
 from jax._src import dispatch
@@ -170,6 +171,45 @@ def _cpp_pjit(fun: Callable, infer_params, static_argnums):
 
   return wraps(fun)(cpp_pjit_f)
 
+class _CppPjitAotCall:
+  def __init__(self, fun: Callable, static_argnums: Any):
+    self._fun = fun
+    self._static_argnums = static_argnums
+
+  def __call__(self, params: CompiledCallParams):
+
+    def aot_cache_miss(*args, **kwargs):
+      # The first invocation for a signature will use the python path. If it is
+      # a correct signature, the later invocations will use the C++ path.
+      # Otherwise, if the signature is wrong, it will be caught by the python
+      # path during the first invocation.
+      outs, out_flat = stages.Compiled.call(params, *args, **kwargs)
+
+      executable = params.executable
+
+      use_fastpath = (
+          isinstance(executable, pxla.MeshExecutable) and
+          isinstance(executable.unsafe_call, pxla.ExecuteReplicated) and
+          not executable.unsafe_call.has_unordered_effects and
+          not executable.unsafe_call.has_host_callbacks and
+          all(isinstance(x, xc.ArrayImpl) for x in out_flat))
+
+      if use_fastpath:
+        out_avals = [o.aval for o in out_flat]
+        out_committed = [o._committed for o in out_flat]
+        fastpath_data = _PjitFastpathData(executable.xla_executable, params.out_tree,
+                                          executable._in_shardings,
+                                          executable._out_shardings,
+                                          out_avals, out_committed)
+      else:
+        fastpath_data = None
+
+      return outs, fastpath_data
+
+    self._cpp_aot_pjit_f = xc._xla.pjit(self._fun, aot_cache_miss,
+                                        self._static_argnums)
+    return self._cpp_aot_pjit_f
+
 
 # TODO(yashkatariya): Add pjit microbenchmarks.
 # in_axis_resources and out_axis_resources can't be None as the default value
@@ -430,11 +470,16 @@ def pjit(fun: Callable,
         params['resource_env'], params['donated_invars'], params['name'],
         in_is_global, always_lower=True)
 
+    if FLAGS.experimental_cpp_pjit and xc._version >= 96:
+      create_cpp_call = _CppPjitAotCall(fun, static_argnums)
+    else:
+      create_cpp_call = None
+
     args_kwargs_in_tree = treedef_tuple([in_tree, tree_flatten({})[1]])
     local_in_avals = args_kwargs_in_tree.unflatten(flat_local_in_avals)
     return stages.Lowered.from_flat_info(
         lowering, args_kwargs_in_tree, local_in_avals, donate_argnums, out_tree,
-        no_kwargs=True)
+        no_kwargs=True, create_cpp_call=create_cpp_call)
 
   wrapped.lower = lower
   return wrapped
@@ -658,7 +703,8 @@ def pjit_check_aval_sharding(
         raise ValueError(f"One of {what_aval} was given the sharding "
                          f"of {s}, which implies that "
                          f"the{global_str} size of its dimension {i} should be "
-                         f"divisible by {size}, but it is equal to {shape[i]}")
+                         f"divisible by {size}, but it is equal to {shape[i]} "
+                         f"(full shape: {shape}) ")
 
 
 class SpecSync(IntEnum):
@@ -947,7 +993,7 @@ class SameDeviceAssignmentTuple:
   def __eq__(self, other):
     if not isinstance(other, SameDeviceAssignmentTuple):
       return False
-    return (all(pxla.are_op_shardings_equal(s._op_sharding, o._op_sharding)
+    return (all(pxla.are_op_shardings_equal(s._op_sharding, o._op_sharding)  # pytype: disable=attribute-error
                 if isinstance(s, OpShardingSharding) and isinstance(o, OpShardingSharding)
                 else s == o
                 for s, o in safe_zip(self.shardings, other.shardings)) and
@@ -1109,7 +1155,7 @@ pxla.spmd_primitive_batchers[pjit_p] = partial(_pjit_batcher, True, None)
 
 def _pjit_batcher_for_sharding(
     s: OpShardingSharding, dim: int, val: Tuple[str, ...], mesh, ndim: int):
-  if not val and xla_extension_version >= 83:
+  if not val:
     new_op = s._op_sharding.clone()
     tad = list(new_op.tile_assignment_dimensions)
     tad.insert(dim, 1)
@@ -1175,9 +1221,7 @@ def _pjit_partial_eval(trace, *in_tracers,
   if config.jax_array:
     residual_shardings = (_UNSPECIFIED,) * num_residuals
   else:
-    # Using fast path to get the device assignment because mesh is used to
-    # create the shardings. So the device assignment is always the same.
-    da = _fast_path_get_device_assignment(it.chain(in_shardings, out_shardings))
+    da = list(resource_env.physical_mesh.devices.flat)
     residual_shardings = (OpShardingSharding.get_replicated(da),) * num_residuals
   # Compute the known outputs
   known_params = dict(
@@ -1191,26 +1235,32 @@ def _pjit_partial_eval(trace, *in_tracers,
       in_positional_semantics=keep_where(in_positional_semantics, known_ins),
       out_positional_semantics=out_positional_semantics)
 
-  # Skip this for Arrays because Arrays support UNSPECIFIED in out_shardings. So
-  # there is no need to find out the residual shardings from XLA here.
-  if not config.jax_array:
-    if num_residuals:
-      in_is_global = _calc_is_global_sequence(
-          known_params['in_positional_semantics'], known_params['in_shardings'])
-      compiled = _pjit_lower(
-          known_params["jaxpr"], known_params["in_shardings"],
-          known_params["out_shardings"], known_params["resource_env"],
-          known_params["donated_invars"], known_params["name"],
-          in_is_global, always_lower=False).compile(
-              _allow_propagation_to_outputs=True,
-              _allow_compile_replicated=False)
-      _, out_op_shardings = _get_op_sharding_from_executable(compiled.xla_executable)
-      residual_op_shardings = tuple(out_op_shardings[-num_residuals:])
-    else:
-      residual_op_shardings = ()
-    residual_shardings = tuple(OpShardingSharding(da, op) for op in residual_op_shardings)
-    known_params['out_shardings'] = (
-        keep_where(out_shardings, known_outs) + residual_shardings)
+  if num_residuals:
+    in_is_global = _calc_is_global_sequence(
+        known_params['in_positional_semantics'], known_params['in_shardings'])
+    compiled = _pjit_lower(
+        known_params["jaxpr"], known_params["in_shardings"],
+        known_params["out_shardings"], known_params["resource_env"],
+        known_params["donated_invars"], known_params["name"],
+        in_is_global, always_lower=False).compile(
+            _allow_propagation_to_outputs=True,
+            _allow_compile_replicated=False)
+    da = compiled._device_assignment
+    _, out_op_sharding_shardings = pxla._get_op_sharding_shardings_from_executable(
+        compiled.xla_executable, da, len(known_jaxpr.in_avals),
+        len(known_jaxpr.out_avals))
+    assert len(out_op_sharding_shardings) == len(known_jaxpr.out_avals), (
+        len(out_op_sharding_shardings), len(known_jaxpr.out_avals))
+    out_op_shardings = [o._to_xla_op_sharding(a.ndim) for o, a in
+                        safe_zip(out_op_sharding_shardings, known_jaxpr.out_avals)]
+    residual_op_shardings = tuple(out_op_shardings[-num_residuals:])
+  else:
+    residual_op_shardings = ()
+  assert len(residual_shardings) == len(residual_op_shardings), (
+      len(residual_shardings), len(residual_op_shardings))
+  residual_shardings = tuple(OpShardingSharding(da, op) for op in residual_op_shardings)
+  known_params['out_shardings'] = (
+      keep_where(out_shardings, known_outs) + residual_shardings)
 
   all_known_outs = pjit_p.bind(
       *(pv.get_known() for pv in in_pvals if pv.is_known()),
@@ -1751,15 +1801,25 @@ def _get_partition_spec(ppspec: Sequence[ParsedPartitionSpec]) -> Sequence[Parti
 def _get_op_sharding_from_executable(
     executable) -> Tuple[Sequence[xc.OpSharding], Sequence[xc.OpSharding]]:
   in_op_shardings: List[xc.OpSharding] = []
-  parameter_shardings_from_xla = executable.hlo_modules()[0].spmd_parameters_shardings
-  if parameter_shardings_from_xla is not None:
-    for s in parameter_shardings_from_xla:
-      in_op_shardings.extend(_get_op_sharding(s))
+  if xla_extension_version >= 100:
+    parameter_shardings_from_xla = executable.get_parameter_shardings()
+    if parameter_shardings_from_xla is not None:
+      in_op_shardings = parameter_shardings_from_xla
+  else:
+    parameter_shardings_from_xla = executable.hlo_modules()[0].spmd_parameters_shardings
+    if parameter_shardings_from_xla is not None:
+      for s in parameter_shardings_from_xla:
+        in_op_shardings.extend(_get_op_sharding(s))
 
   out_op_shardings: List[xc.OpSharding] = []
-  output_shardings_from_xla = executable.hlo_modules()[0].spmd_output_sharding
-  if output_shardings_from_xla is not None:
-    out_op_shardings = _get_op_sharding(output_shardings_from_xla)  # type: ignore
+  if xla_extension_version >= 100:
+    output_shardings_from_xla = executable.get_output_shardings()
+    if output_shardings_from_xla is not None:
+      out_op_shardings = output_shardings_from_xla
+  else:
+    output_shardings_from_xla = executable.hlo_modules()[0].spmd_output_sharding
+    if output_shardings_from_xla is not None:
+      out_op_shardings = _get_op_sharding(output_shardings_from_xla)  # type: ignore
 
   return in_op_shardings, out_op_shardings
 
@@ -1781,97 +1841,3 @@ def _get_pspec_from_executable(
   out_partition_spec = _get_partition_spec(out_ppspec)
   in_partition_spec = _get_partition_spec(in_ppspec)
   return tuple(in_partition_spec), tuple(out_partition_spec)
-
-
-def host_local_array_to_global_array(local_inputs, global_mesh, pspecs):
-  """Converts a host local value to a globally sharded `jax.Array`.
-
-  You can use this function to transition to `jax.Array`. Using `jax.Array` with
-  `pjit` has the same semantics of using GDA with pjit i.e. all `jax.Array`
-  inputs to pjit should be globally shaped.
-
-  If you are currently passing host local values to pjit, you can use this
-  function to convert your host local values to global Arrays and then pass that
-  to pjit.
-
-  Example usage:
-
-  ```
-  global_inputs = jax.experimental.pjit.host_local_array_to_global_array(
-    host_local_inputs, global_mesh, in_pspecs)
-
-  with mesh:
-    global_out = pjitted_fun(global_inputs)
-
-  host_local_output = jax.experimental.pjit.global_array_to_host_local_array(
-    global_out, mesh, out_pspecs)
-  ```
-
-  Args:
-    local_inputs: A Pytree of host local values.
-    global_mesh: The global mesh.
-    pspecs: A Pytree of PartitionSpecs.
-  """
-  def _convert(arr, pspec):
-    if isinstance(arr, array.ArrayImpl) and isinstance(arr.sharding, PmapSharding):
-      arr = np.array(arr)
-    local_sharding = MeshPspecSharding(global_mesh.local_mesh, pspec)
-    arrays = [
-        device_put(arr[index], d)
-        for d, index in local_sharding.devices_indices_map(arr.shape).items()
-    ]
-    global_aval = global_mesh._local_to_global(
-        pxla._get_array_mapping(pspec),
-        core.ShapedArray(arr.shape, arrays[0].dtype))
-    return array.ArrayImpl(global_aval, MeshPspecSharding(global_mesh, pspec),
-                           arrays, committed=True)
-
-  flattened_inps, in_tree = tree_flatten(local_inputs)
-  in_pspecs = flatten_axis_resources(
-      'input pspecs', in_tree, pspecs, tupled_args=True)
-  out = tree_map(_convert, tuple(flattened_inps), in_pspecs)
-  return tree_unflatten(in_tree, out)
-
-
-def global_array_to_host_local_array(global_inputs, global_mesh, pspecs):
-  """Converts a global `jax.Array` to a host local `jax.Array`.
-
-  You can use this function to transition to `jax.Array`. Using `jax.Array` with
-  `pjit` has the same semantics of using GDA with pjit i.e. all `jax.Array`
-  inputs to pjit should be globally shaped and the output from `pjit` will also
-  be globally shaped `jax.Array`s
-
-  You can use this function to convert the globally shaped `jax.Array` output
-  from pjit to host local values again so that the transition to jax.Array can
-  be a mechanical change.
-
-  Example usage:
-
-  ```
-  global_inputs = jax.experimental.pjit.host_local_array_to_global_array(
-    host_local_inputs, global_mesh, in_pspecs)
-
-  with mesh:
-    global_out = pjitted_fun(global_inputs)
-
-  host_local_output = jax.experimental.pjit.global_array_to_host_local_array(
-    global_out, mesh, out_pspecs)
-  ```
-
-  Args:
-    global_inputs: A Pytree of global `jax.Array`s.
-    global_mesh: The global mesh.
-    pspecs: A Pytree of PartitionSpecs.
-  """
-  def _convert(arr, pspec):
-    local_aval = global_mesh._global_to_local(
-        pxla._get_array_mapping(pspec), arr.aval)
-    return array.ArrayImpl(
-        local_aval, MeshPspecSharding(global_mesh.local_mesh, pspec),
-        arr._arrays, committed=True)
-
-  flattened_inps, out_tree = tree_flatten(global_inputs)
-  out_pspecs = flatten_axis_resources(
-      'output pspecs', out_tree, pspecs, tupled_args=True)
-  out = tree_map(_convert, tuple(flattened_inps), out_pspecs)
-  return tree_unflatten(out_tree, out)
