@@ -29,14 +29,12 @@ from typing import (Any, Callable, Dict, Iterator, List, NamedTuple, Optional,
 from typing_extensions import Protocol
 import warnings
 
-import jax
 from jax import core
 from jax import linear_util as lu
 from jax._src import ad_util
 from jax._src import device_array
 from jax._src import dtypes
-from jax._src.lib import mlir_api_version
-from jax._src.lib import version as jaxlib_version
+from jax._src.lib import mlir_api_version, xla_extension_version
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import chlo
 from jax._src.lib.mlir.dialects import mhlo
@@ -144,8 +142,8 @@ def _array_ir_types(aval: Union[core.ShapedArray, core.DShapedArray]
   return (ir.RankedTensorType.get(aval.shape, dtype_to_ir_type(aval.dtype)),)
 
 def _dynamic_array_ir_types(aval: core.ShapedArray) -> Sequence[ir.Type]:
-  # in the MHLO builder, -1 indicates a '?' axis size
-  shape = [d if type(d) is int else -1 for d in aval.shape]
+  dyn_size = ir.ShapedType.get_dynamic_size() if mlir_api_version >= 35 else -1
+  shape = [d if type(d) is int else dyn_size for d in aval.shape]
   return (ir.RankedTensorType.get(shape, dtype_to_ir_type(aval.dtype)),)
 
 ir_type_handlers: Dict[Type[core.AbstractValue],
@@ -323,7 +321,7 @@ def _source_info_to_location(
     loc = ir.Location.unknown()
   else:
     loc = ir.Location.file(xla._get_canonical_source_file(frame),
-                           frame.line_num, 1)
+                           frame.start_line, frame.start_column)
   loc = ir.Location.name(eqn_str, childLoc=loc)
   # TODO(phawkins): also include primitive.name as the operator type.
   return loc
@@ -336,10 +334,7 @@ def make_ir_context() -> ir.Context:
   """Creates an MLIR context suitable for JAX IR."""
   context = ir.Context()
   mhlo.register_mhlo_dialect(context)
-  if mlir_api_version < 33:
-    chlo.register_chlo_dialect(context)
-  else:
-    chlo.register_dialect(context)
+  chlo.register_dialect(context)
   return context
 
 
@@ -571,6 +566,12 @@ class LoweringResult(NamedTuple):
   host_callbacks: List[Any]
 
 
+if xla_extension_version >= 102:
+  _platforms_with_donation = ["cpu", "cuda", "rocm", "tpu"]
+else:
+  _platforms_with_donation = ["cuda", "rocm", "tpu"]
+
+
 def lower_jaxpr_to_module(
     module_name: str,
     jaxpr: core.ClosedJaxpr,
@@ -609,8 +610,7 @@ def lower_jaxpr_to_module(
         out_aval, = out_aval.dtype._rules.physical_avals(out_aval)
       out_avals.append(sharded_aval(out_aval, out_sharding))
 
-  platforms_with_donation = ("cuda", "rocm", "tpu")
-  if platform in platforms_with_donation:
+  if platform in _platforms_with_donation:
     input_output_aliases, donated_args = _set_up_aliases(
         in_avals, out_avals, donated_args)
   if any(eff not in lowerable_effects for eff in jaxpr.effects):
@@ -620,7 +620,7 @@ def lower_jaxpr_to_module(
     unused_donations = [str(a) for a, d in zip(in_avals, donated_args)
                         if d]
     msg = "See an explanation at https://jax.readthedocs.io/en/latest/faq.html#buffer-donation."
-    if platform not in platforms_with_donation:
+    if platform not in _platforms_with_donation:
       msg = f"Donation is not implemented for {platform}.\n{msg}"
     warnings.warn(f"Some donated buffers were not usable: {', '.join(unused_donations)}.\n{msg}")
 
@@ -977,13 +977,14 @@ def jaxpr_subcomp(ctx: ModuleContext, jaxpr: core.Jaxpr,
   Assumes that an MLIR context, location, and insertion point are set.
   """
   assert ctx.platform != "gpu"
-  def read(v: core.Var) -> Sequence[ir.Value]:
+  def read(v: core.Atom) -> Sequence[ir.Value]:
     if type(v) is core.Literal:
       return ir_constants(v.val, canonicalize_types=True)
     else:
+      assert isinstance(v, core.Var)
       return env[v]
 
-  def aval(v: core.Var) -> core.AbstractValue:
+  def aval(v: core.Atom) -> core.AbstractValue:
     if type(v) is core.Literal:
       return xla.abstractify(v.val)
     else:
@@ -1549,9 +1550,6 @@ def emit_python_callback(
     ) -> Tuple[List[ir.Value], Any, Any]:
   """Emits MHLO that calls back to a provided Python function."""
   platform = ctx.module_context.platform
-  if platform in {"tpu"} and jaxlib_version < (0, 3, 15):
-    raise ValueError(
-        "`EmitPythonCallback` on TPU only supported on jaxlib >= 0.3.15")
   if platform not in {"cpu", "cuda", "rocm", "tpu"}:
     raise ValueError(
         f"`EmitPythonCallback` not supported on {platform} backend.")
@@ -1599,9 +1597,10 @@ def emit_python_callback(
     ]
     operands = [token, *operands]
     result_types = [token_type()[0], *result_types]
+  # TODO(jakevdp) remove ignore statement once jaxlib>0.3.22 is released.
   callback_descriptor, keepalive = (
       backend.get_emit_python_callback_descriptor(_wrapped_callback,
-                                                  operand_shapes,
+                                                  operand_shapes,  # type: ignore[arg-type]
                                                   result_shapes))
   descriptor_operand = ir_constant(
       callback_descriptor, canonicalize_types=False)
